@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.30 2002/02/28 20:33:19 rjs3 Exp $ */
+/* $Id: acconfig.h,v 1.34 2002/04/15 19:35:49 rjs3 Exp $ */
 /* 
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -77,6 +77,9 @@
 /* do we support LISTEXT? */
 #undef ENABLE_LISTEXT
 
+/* do we support ANNOTATEMORE? */
+#undef ENABLE_ANNOTATEMORE
+
 /* do we support XNETSCAPE */
 #undef ENABLE_X_NETSCAPE_HACK
 
@@ -104,6 +107,9 @@
 /* alternative to /dev/urandom? */
 #undef EGD_SOCKET
 
+/* do we have zephyr? */
+#undef HAVE_ZEPHYR
+
 /* where should we put state information? */
 #undef STATEDIR
 
@@ -134,6 +140,14 @@
 /* do we have rlim_t? */
 #undef HAVE_RLIM_T
 
+/* do we have fdatasync */
+#undef HAVE_FDATASYNC
+
+/* This allows us to work even when we don't have an fdatasync */
+#ifndef HAVE_FDATASYNC
+#define fdatasync(fd) fsync(fd)
+#endif
+
 /* Database Backends that are configurable */
 #undef CONFIG_DB_DUPLICATE
 #undef CONFIG_DB_MBOX
@@ -170,6 +184,7 @@ typedef int rlim_t;
 
 /* getaddrinfo things */
 #include <netdb.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 
 #ifndef HAVE_GETADDRINFO
