@@ -61,6 +61,7 @@
 #include "util.h"
 #include "imap_err.h"
 #include "mupdate_err.h"
+#include "prot.h" /* for PROT_BUFSIZE */
 
 extern int errno;
 
@@ -78,12 +79,13 @@ static int nconfiglist;
 /* variables accessible to the external world */
 const char *config_filename;     /* filename of configuration file */
 
-const char *config_dir;		 /* ie /var/imap */
-const char *config_defpartition; /* /var/spool/imap */
-const char *config_newsspool;	 /* /var/spool/news */
-const char *config_servername;	 /* gethostname() */
-int config_hashimapspool;	 /* f */
+const char *config_dir;		           /* ie /var/imap */
+const char *config_defpartition;           /* /var/spool/imap */
+const char *config_newsspool;	           /* /var/spool/news */
+const char *config_servername;	           /* gethostname() */
+const char *config_mupdate_server = NULL;  /* NULL */
 
+int config_hashimapspool;	           /* f */
 
 static void config_read(const char *alt_config);
 
@@ -143,6 +145,8 @@ int config_init(const char *alt_config, const char *ident)
 	gethostname((char *) config_servername, 256);
     }
 
+    config_mupdate_server = config_getstring("mupdate_server", NULL);
+
     return 0;
 }
 
@@ -339,7 +343,7 @@ sasl_security_properties_t *mysasl_secprops(int flags)
 {
     static sasl_security_properties_t ret;
 
-    ret.maxbufsize = 4000;
+    ret.maxbufsize = PROT_BUFSIZE;
     ret.min_ssf = config_getint("sasl_minimum_layer", 0);	
 				/* minimum allowable security strength */
     ret.max_ssf = config_getint("sasl_maximum_layer", 256);
