@@ -82,7 +82,6 @@
 #include "duplicate.h"
 
 #define DB (CONFIG_DB_DUPLICATE)
-#define FNAME_DELIVERDB "/deliver.db"
 
 static struct db *dupdb = NULL;
 static int duplicate_dbopen = 0;
@@ -162,7 +161,7 @@ time_t duplicate_check(char *id, int idlen, char *to, int tolen)
 	mark = 0;
     }
 
-    syslog(LOG_DEBUG, "duplicate_check: %-40s %-20s %d",
+    syslog(LOG_DEBUG, "duplicate_check: %-40s %-20s %ld",
 	   buf, buf+idlen+1, mark);
 
     return mark;
@@ -188,7 +187,7 @@ void duplicate_mark(char *id, int idlen, char *to, int tolen, time_t mark)
 		      (char *) &mark, sizeof(mark), NULL);
     } while (r == CYRUSDB_AGAIN);
 
-    syslog(LOG_DEBUG, "duplicate_mark: %-40s %-20s %d",
+    syslog(LOG_DEBUG, "duplicate_mark: %-40s %-20s %ld",
 	   buf, buf+idlen+1, mark);
 
     return;
@@ -225,7 +224,7 @@ static int prune_cb(void *rock, const char *id, int idlen,
     prock->deletions++;
 
     do {
-	r = DB->delete(prock->db, id, idlen, NULL);
+	r = DB->delete(prock->db, id, idlen, NULL, 0);
     } while (r == CYRUSDB_AGAIN);
 
 
