@@ -42,10 +42,11 @@
 #ifndef DUPLICATE_H
 #define DUPLICATE_H
 
- /* we have binary data and \0 -> can't use flat */
-#define CONFIG_DB_DUPLICATE (&cyrusdb_db3_nosync)
 #define DUPLICATE_RECOVER 0x01
 
+/* name of the duplicate delivery database */
+#define FNAME_DELIVERDB "/deliver.db"
+
 int duplicate_init(char*, int);
 
 time_t duplicate_check(char *id, int idlen, char *to, int tolen);
