@@ -1,6 +1,6 @@
 /* lmtpd.c -- Program to deliver mail to a mailbox
  *
- * $Id: lmtpd.c,v 1.88 2002/03/07 17:55:28 rjs3 Exp $
+ * $Id: lmtpd.c,v 1.96 2002/04/15 14:42:17 rjs3 Exp $
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -122,7 +122,7 @@ typedef struct script_data {
 
 /* forward declarations */
 int deliver_mailbox(struct protstream *msg,
-		    struct stagemsg **stage,
+		    struct stagemsg *stage,
 		    unsigned size,
 		    char **flag,
 		    int nflags,
@@ -142,7 +142,11 @@ static char *generate_notify(message_data_t *m);
 
 void shut_down(int code);
 
-struct lmtp_func mylmtp = { &deliver, &verify_user, &shut_down, 0, 1, 0 };
+static FILE *spoolfile(message_data_t *msgdata);
+static void removespool(message_data_t *msgdata);
+
+struct lmtp_func mylmtp = { &deliver, &verify_user, &shut_down,
+			    &spoolfile, &removespool, 0, 1, 0 };
 
 static void logdupelem();
 static void usage();
@@ -172,11 +176,14 @@ static struct namespace lmtpd_namespace;
 /* should we allow users to proxy?  return SASL_OK if yes,
    SASL_BADAUTH otherwise */
 static int mysasl_authproc(sasl_conn_t *conn,
-			   void *context,
-			   const char *requested_user, unsigned rlen,
-			   const char *auth_identity, unsigned alen,
-			   const char *def_realm, unsigned urlen,
-			   struct propctx *propctx)
+			   void *context __attribute__((unused)),
+			   const char *requested_user __attribute__((unused)),
+			   unsigned rlen __attribute__((unused)),
+			   const char *auth_identity, 
+			   unsigned alen __attribute__((unused)),
+			   const char *def_realm __attribute__((unused)),
+			   unsigned urlen __attribute__((unused)),
+			   struct propctx *propctx __attribute__((unused)))
 {
     const char *val;
     char *realm;
@@ -226,7 +233,9 @@ static struct sasl_callback mysasl_cb[] = {
 };
 
 
-int service_init(int argc, char **argv, char **envp)
+int service_init(int argc __attribute__((unused)), 
+		 char **argv __attribute__((unused)), 
+		 char **envp __attribute__((unused)))
 {
     int r;
 
@@ -294,7 +303,8 @@ int service_init(int argc, char **argv, char **envp)
 /*
  * run for each accepted connection
  */
-int service_main(int argc, char **argv, char **envp)
+int service_main(int argc, char **argv, 
+		 char **envp __attribute__((unused)))
 {
     int opt;
 
@@ -594,7 +604,9 @@ int send_forward(char *forwardto, char *return_path, struct protstream *file)
 
 
 static
-int sieve_redirect(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int sieve_redirect(void *ac, 
+		   void *ic __attribute__((unused)), 
+		   void *sc, void *mc, const char **errmsg)
 {
     sieve_redirect_context_t *rc = (sieve_redirect_context_t *) ac;
     script_data_t *sd = (script_data_t *) sc;
@@ -630,7 +642,10 @@ int sieve_redirect(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 }
 
 static
-int sieve_discard(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int sieve_discard(void *ac __attribute__((unused)), 
+		  void *ic __attribute__((unused)), 
+		  void *sc, void *mc, 
+		  const char **errmsg __attribute__((unused)))
 {
     script_data_t *sd = (script_data_t *) sc;
     message_data_t *md = ((mydata_t *) mc)->m;
@@ -654,7 +669,9 @@ int sieve_discard(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 }
 
 static
-int sieve_reject(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int sieve_reject(void *ac, 
+		 void *ic __attribute__((unused)), 
+		 void *sc, void *mc, const char **errmsg)
 {
     sieve_reject_context_t *rc = (sieve_reject_context_t *) ac;
     script_data_t *sd = (script_data_t *) sc;
@@ -687,7 +704,11 @@ int sieve_reject(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 }
 
 static
-int sieve_fileinto(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int sieve_fileinto(void *ac, 
+		   void *ic __attribute__((unused)),
+		   void *sc, 
+		   void *mc __attribute__((unused)), 
+		   const char **errmsg __attribute__((unused)))
 {
     sieve_fileinto_context_t *fc = (sieve_fileinto_context_t *) ac;
     script_data_t *sd = (script_data_t *) sc;
@@ -700,7 +721,7 @@ int sieve_fileinto(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
     if (!sd->authstate)
 	return SIEVE_FAIL;
 
-    ret = deliver_mailbox(md->data, &mdata->stage, md->size,
+    ret = deliver_mailbox(md->data, mdata->stage, md->size,
 			  fc->imapflags->flag, fc->imapflags->nflags,
                           sd->username, sd->authstate, md->id,
                           sd->username, mdata->notifyheader,
@@ -718,7 +739,9 @@ int sieve_fileinto(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 }
 
 static
-int sieve_keep(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int sieve_keep(void *ac, 
+	       void *ic __attribute__((unused)),
+	       void *sc, void *mc, const char **errmsg)
 {
     sieve_keep_context_t *kc = (sieve_keep_context_t *) ac;
     script_data_t *sd = (script_data_t *) sc;
@@ -732,7 +755,7 @@ int sieve_keep(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 	strcpy(namebuf, lmtpd_namespace.prefix[NAMESPACE_INBOX]);
 	strcat(namebuf, sd->mailboxname);
 
-	ret = deliver_mailbox(md->data, &mydata->stage, md->size,
+	ret = deliver_mailbox(md->data, mydata->stage, md->size,
 			      kc->imapflags->flag, kc->imapflags->nflags,
 			      mydata->authuser, mydata->authstate, md->id,
 			      sd->username, mydata->notifyheader,
@@ -745,7 +768,7 @@ int sieve_keep(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 
 	strcpy(namebuf, "INBOX");
 
-	ret = deliver_mailbox(md->data, &mydata->stage, md->size,
+	ret = deliver_mailbox(md->data, mydata->stage, md->size,
 			      kc->imapflags->flag, kc->imapflags->nflags,
 			      sd->username, sd->authstate, md->id,
 			      sd->username, mydata->notifyheader,
@@ -762,26 +785,36 @@ int sieve_keep(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 }
 
 static int sieve_notify(void *ac,
-			void *interp_context, 
+			void *interp_context __attribute__((unused)), 
 			void *script_context,
-			void *mc,
-			const char **errmsg)
+			void *mc __attribute__((unused)),
+			const char **errmsg __attribute__((unused)))
 {
-    sieve_notify_context_t *nc = (sieve_notify_context_t *) ac;
-    script_data_t *sd = (script_data_t *) script_context;
+    const char *notifier = config_getstring("sievenotifier", NULL);
 
-    snmp_increment(SIEVE_NOTIFY, 1);
+    if (notifier) {
+	sieve_notify_context_t *nc = (sieve_notify_context_t *) ac;
+	script_data_t *sd = (script_data_t *) script_context;
+	int nopt = 0;
 
-    notify("SIEVE",
-	   nc->priority,
-	   sd->username,
-	   NULL,
-	   nc->message);
+	snmp_increment(SIEVE_NOTIFY, 1);
+
+	/* count options */
+	while (nc->options[nopt]) nopt++;
+
+	notify(nc->method ? nc->method : notifier,
+	       "SIEVE", nc->priority, sd->username, NULL,
+	       nopt, nc->options, nc->message);
+    }
     
     return SIEVE_OK;
 }
 
-int autorespond(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int autorespond(void *ac, 
+		void *ic __attribute__((unused)), 
+		void *sc,
+		void *mc __attribute__((unused)),
+		const char **errmsg __attribute__((unused)))
 {
     sieve_autorespond_context_t *arc = (sieve_autorespond_context_t *) ac;
     script_data_t *sd = (script_data_t *) sc;
@@ -816,7 +849,9 @@ int autorespond(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
     return ret;
 }
 
-int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
+int send_response(void *ac, 
+		  void *ic __attribute__((unused)), 
+		  void *sc, void *mc, const char **errmsg)
 {
     FILE *sm;
     const char *smbuf[10];
@@ -910,7 +945,9 @@ sieve_vacation_t vacation = {
 static char *markflags[] = { "\\flagged" };
 static sieve_imapflags_t mark = { markflags, 1 };
 
-int sieve_parse_error_handler(int lineno, const char *msg, void *ic, void *sc)
+int sieve_parse_error_handler(int lineno, const char *msg, 
+			      void *ic __attribute__((unused)),
+			      void *sc)
 {
     script_data_t *sd = (script_data_t *) sc;
     
@@ -920,7 +957,9 @@ int sieve_parse_error_handler(int lineno, const char *msg, void *ic, void *sc)
     return SIEVE_OK;
 }
 
-int sieve_execute_error_handler(const char *msg, void *ic, void *sc, void *mc)
+int sieve_execute_error_handler(const char *msg, 
+				void *ic  __attribute__((unused)), 
+				void *sc, void *mc)
 {
     script_data_t *sd = (script_data_t *) sc;
     message_data_t *md = ((mydata_t *) mc)->m;
@@ -1069,7 +1108,7 @@ usage()
  * pass acloverride
  */
 int deliver_mailbox(struct protstream *msg,
-		    struct stagemsg **stage,
+		    struct stagemsg *stage,
 		    unsigned size,
 		    char **flag,
 		    int nflags,
@@ -1102,13 +1141,13 @@ int deliver_mailbox(struct protstream *msg,
 
     if (!r) {
 	r = append_setup(&as, namebuf, MAILBOX_FORMAT_NORMAL,
-			 NULL, authstate, acloverride ? 0 : ACL_POST, 
+			 authuser, authstate, acloverride ? 0 : ACL_POST, 
 			 quotaoverride ? -1 : 0);
     }
 
     if (!r) {
 	prot_rewind(msg);
-	if (singleinstance && stage) {
+	if (stage) {
 	    r = append_fromstage(&as, msg, size, now, 
 				 (const char **) flag, nflags, stage);
 	} else {
@@ -1120,9 +1159,13 @@ int deliver_mailbox(struct protstream *msg,
     }
 
     if (!r && user) {
-	/* do we want to replace user.XXX with INBOX? */
-	notify("MAIL", mailboxname, user, mailboxname, 
-	       notifyheader ? notifyheader : "");
+	const char *notifier = config_getstring("mailnotifier", NULL);
+
+	if (notifier) {
+	    /* do we want to replace user.XXX with INBOX? */
+	    notify(notifier, "MAIL", NULL, user, mailboxname, 0, NULL,
+		   notifyheader ? notifyheader : "");
+	}
     }
 
     if (!r && dupelim && id) duplicate_mark(id, strlen(id), 
@@ -1144,7 +1187,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 
     /* create 'mydata', our per-delivery data */
     mydata.m = msgdata;
-    mydata.stage = NULL;
+    mydata.stage = (struct stagemsg *) msg_getrock(msgdata);
     mydata.notifyheader = generate_notify(msgdata);
     mydata.authuser = authuser;
     mydata.authstate = authstate;
@@ -1164,7 +1207,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	    strcpy(namebuf, lmtpd_namespace.prefix[NAMESPACE_SHARED]);
 	    strcat(namebuf, plus);
 	    r = deliver_mailbox(msgdata->data, 
-				&mydata.stage,
+				mydata.stage,
 				msgdata->size, 
 				NULL, 0,
 				mydata.authuser, mydata.authstate,
@@ -1241,7 +1284,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 		strcat(namebuf, plus);
 		
 		r = deliver_mailbox(msgdata->data, 
-				    &mydata.stage, 
+				    mydata.stage, 
 				    msgdata->size, 
 				    NULL, 0, 
 				    mydata.authuser, mydata.authstate,
@@ -1255,7 +1298,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 		
 		/* ignore ACL's trying to deliver to INBOX */
 		r = deliver_mailbox(msgdata->data, 
-				    &mydata.stage,
+				    mydata.stage,
 				    msgdata->size, 
 				    NULL, 0, 
 				    mydata.authuser, mydata.authstate,
@@ -1308,7 +1351,10 @@ void fatal(const char* s, int code)
     if(deliver_out) {
 	prot_printf(deliver_out,"421 4.3.0 lmtpd: %s\r\n", s);
 	prot_flush(deliver_out);
+    } else {
+	syslog(LOG_ERR, "FATAL: %s", s);
     }
+    
     /* shouldn't return */
     shut_down(code);
 
@@ -1384,8 +1430,8 @@ char *generate_notify(message_data_t *m)
 {
     const char **body;
     char *ret = NULL;
-    int len = 0;
-    int pos = 0;
+    unsigned int len = 0;
+    unsigned int pos = 0;
     int i;
 
     for (i = 0; notifyheaders[i]; i++) {
@@ -1413,3 +1459,74 @@ char *generate_notify(message_data_t *m)
 
     return ret;
 }
+
+FILE *spoolfile(message_data_t *msgdata)
+{
+    /* if we have a single recipient OR are using single-instance store,
+     * spool to the stage of the first recipient
+     */
+    if ((msg_getnumrcpt(msgdata) == 1) || singleinstance) {
+	int r = 0;
+	char *rcpt, *plus, *user = NULL;
+	char namebuf[MAX_MAILBOX_PATH], mailboxname[MAX_MAILBOX_PATH];
+	time_t now = time(NULL);
+
+	/* build the mailboxname from the recipient address */
+	rcpt = xstrdup(msg_getrcpt(msgdata, 0));
+	plus = strchr(rcpt, '+');
+	if (plus) *plus++ = '\0';
+
+	/* case 1: shared mailbox request */
+	if (plus && !strcmp(rcpt, BB)) {
+	    strcpy(namebuf, lmtpd_namespace.prefix[NAMESPACE_SHARED]);
+	    strcat(namebuf, plus);
+	}
+
+	/* case 2: ordinary user */
+	else if (!strchr(rcpt, lmtpd_namespace.hier_sep) &&
+	         strlen(rcpt) + 30 <= MAX_MAILBOX_PATH) {
+	    user = rcpt;
+
+	    /* assume delivery to INBOX for now */
+	    strcpy(namebuf, "INBOX");
+	}
+
+	/* case 3: unable to handle rcpt */
+	else {
+	    /* force error and we'll fallback to using /tmp */
+	    r = 1;
+	}
+
+	if (!r) {
+	    /* Translate any separators in user */
+	    if (user) mboxname_hiersep_tointernal(&lmtpd_namespace, user);
+
+	    r = (*lmtpd_namespace.mboxname_tointernal)(&lmtpd_namespace,
+						       namebuf,
+						       user, mailboxname);
+	}
+
+	free(rcpt);
+
+	if (!r) {
+	    FILE *f;
+	    struct stagemsg *stage = NULL;
+
+	    /* setup stage for later use by deliver() */
+	    f = append_newstage(mailboxname, now, &stage);
+	    msg_setrock(msgdata, (void*) stage);
+
+	    return f;
+	}
+    }
+
+    /* spool to /tmp (no single-instance store) */
+    return tmpfile();
+}
+
+void removespool(message_data_t *msgdata)
+{
+    struct stagemsg *stage = (struct stagemsg *) msg_getrock(msgdata);
+
+    append_removestage(stage);
+}
