@@ -1,5 +1,5 @@
 /* lmtpengine.c: LMTP protocol engine
- * $Id: lmtpengine.c,v 1.65 2002/03/07 17:55:28 rjs3 Exp $
+ * $Id: lmtpengine.c,v 1.72 2002/04/17 20:31:52 ken3 Exp $
  *
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -88,6 +88,7 @@
 #include "lmtpengine.h"
 #include "lmtpstats.h"
 #include "tls.h"
+#include "telemetry.h"
 
 #define RCPT_GROW 30
 
@@ -637,33 +638,49 @@ char *buf;
     }
 }
 
-/* copies the message from fin to fout, massaging accordingly: mostly
- * newlines are fiddled. "." terminates */
+/* copies the message from fin to fout, massaging accordingly: 
+   . newlines are fiddled to \r\n
+   . "." terminates 
+   . embedded NULs are rejected
+   . bare \r are removed
+*/
 static int copy_msg(struct protstream *fin, FILE *fout)
 {
     char buf[8192], *p;
+    int r = 0;
 
     while (prot_fgets(buf, sizeof(buf)-1, fin)) {
 	p = buf + strlen(buf) - 1;
-	if (p <= buf || (p[0] == '\n' && p[-1] != '\r')) {
-	    /* either a \0 by itself or a \n without a \r */
+	if (p < buf) {
+	    /* buffer start with a \0 */
+	    r = IMAP_MESSAGE_CONTAINSNULL;
+	    continue; /* need to eat the rest of the message */
+	}
+	else if (buf[0] == '\r' && buf[1] == '\0') {
+	    /* The message contained \r\0, and fgets is confusing us. */
+	    r = IMAP_MESSAGE_CONTAINSNULL;
+	    continue; /* need to eat the rest of the message */
+	}
+	else if (p[0] == '\r') {
+	    /*
+	     * We were unlucky enough to get a CR just before we ran
+	     * out of buffer--put it back.
+	     */
+	    prot_ungetc('\r', fin);
+	    *p = '\0';
+	}
+	else if (p[0] == '\n' && p[-1] != '\r') {
+	    /* found an \n without a \r */
 	    p[0] = '\r';
 	    p[1] = '\n';
 	    p[2] = '\0';
-	} else if (*p == '\r') {
-	    if (buf[0] == '\r' && buf[1] == '\0') {
-		/* The message contained \r\0, and fgets is confusing us.
-		   XXX ignored
-		   */
-	    } else {
-		/*
-		 * We were unlucky enough to get a CR just before we ran
-		 * out of buffer--put it back.
-		 */
-		prot_ungetc('\r', fin);
-		*p = '\0';
-	    }
 	}
+	else if (p[0] != '\n') {
+	    /* line contained a \0 not at the end */
+	    r = IMAP_MESSAGE_CONTAINSNULL;
+	    continue;
+	}
+
 	/* Remove any lone CR characters */
 	while ((p = strchr(buf, '\r')) && p[1] != '\n') {
 	    strcpy(p, p+1);
@@ -685,7 +702,7 @@ static int copy_msg(struct protstream *fin, FILE *fout)
     return IMAP_IOERROR;
 
  lmtpdot:
-    return 0;
+    return r;
 }
 
 /* take a list of headers, pull the first one out and return it in
@@ -708,7 +725,14 @@ enum {
 };
 
 /* we don't have to worry about dotstuffing here, since it's illegal
-   for a header to begin with a dot! */
+   for a header to begin with a dot!
+
+   returns 0 on success, filling in 'headname' and 'contents' with a static
+   pointer (blech).
+   on end of headers, returns 0 with NULL 'headname' and NULL 'contents'
+
+   on error, returns < 0
+*/
 static int parseheader(struct protstream *fin, FILE *fout, 
 		       char **headname, char **contents) {
     int c;
@@ -716,6 +740,8 @@ static int parseheader(struct protstream *fin, FILE *fout,
     static int namelen = 0, bodylen = 0;
     int off = 0;
     state s = NAME_START;
+    int r = 0;
+    int reject8bit = config_getswitch("reject8bit", 0);
 
     if (namelen == 0) {
 	namelen += NAMEINC;
@@ -727,13 +753,26 @@ static int parseheader(struct protstream *fin, FILE *fout,
     }
 
     /* there are two ways out of this loop, both via gotos:
-       either we successfully read a character (got_header)
+       either we successfully read a header (got_header)
        or we hit an error (ph_error) */
     while ((c = prot_getc(fin)) != EOF) { /* examine each character */
 	switch (s) {
 	case NAME_START:
+	    if (c == '.') {
+		int peek;
+
+		peek = prot_getc(fin);
+		prot_ungetc(peek, fin);
+		
+		if (peek == '\r' || peek == '\n') {
+		    /* just reached the end of message */
+		    r = IMAP_MESSAGE_NOBLANKLINE;
+		    goto ph_error;
+		}
+	    }
 	    if (c == '\r' || c == '\n') {
-		/* no header here! */
+		/* just reached the end of headers */
+		r = 0;
 		goto ph_error;
 	    }
 	    /* field-name      =       1*ftext
@@ -743,6 +782,7 @@ static int parseheader(struct protstream *fin, FILE *fout,
 				       ;  ":". */
 	    if (!((c >= 33 && c <= 57) || (c >= 59 && c <= 126))) {
 		/* invalid header name */
+		r = IMAP_MESSAGE_BADHEADER;
 		goto ph_error;
 	    }
 	    name[0] = tolower(c);
@@ -757,6 +797,7 @@ static int parseheader(struct protstream *fin, FILE *fout,
 		break;
 	    }
 	    if (!((c >= 33 && c <= 57) || (c >= 59 && c <= 126))) {
+		r = IMAP_MESSAGE_BADHEADER;
 		goto ph_error;
 	    }
 	    name[off++] = tolower(c);
@@ -775,6 +816,7 @@ static int parseheader(struct protstream *fin, FILE *fout,
 		    fputc(c, fout);
 		    c = prot_getc(fin);
 		}
+		r = IMAP_MESSAGE_BADHEADER;
 		goto ph_error;
 	    }
 	    break;
@@ -809,6 +851,18 @@ static int parseheader(struct protstream *fin, FILE *fout,
 		/* ignore this whitespace, but we'll copy all the rest in */
 		break;
 	    } else {
+		if (c >= 0x80) {
+		    if (reject8bit) {
+			/* We have been configured to reject all mail of this
+			   form. */
+			r = IMAP_MESSAGE_CONTAINS8BIT;
+			goto ph_error;
+		    } else {
+			/* We have been configured to munge all mail of this
+			   form. */
+			c = 'X';
+		    }
+		}
 		/* just an ordinary character */
 		body[off++] = c;
 		if (off >= bodylen - 3) {
@@ -832,7 +886,7 @@ static int parseheader(struct protstream *fin, FILE *fout,
     /* and we didn't get a header */
     if (headname != NULL) *headname = NULL;
     if (contents != NULL) *contents = NULL;
-    return -1;
+    return r;
 
  got_header:
     if (headname != NULL) *headname = xstrdup(name);
@@ -843,12 +897,18 @@ static int parseheader(struct protstream *fin, FILE *fout,
 
 static int fill_cache(struct protstream *fin, FILE *fout, message_data_t *m)
 {
+    int r = 0;
+
     /* let's fill that header cache */
     for (;;) {
 	char *name, *body;
 	int cl, clinit;
 
-	if (parseheader(fin, fout, &name, &body) < 0) {
+	if ((r = parseheader(fin, fout, &name, &body)) < 0) {
+	    break;
+	}
+	if (!name) {
+	    /* reached the end of headers */
 	    break;
 	}
 
@@ -888,7 +948,16 @@ static int fill_cache(struct protstream *fin, FILE *fout, message_data_t *m)
 	m->cache[cl]->contents[m->cache[cl]->ncontents] = NULL;
     }
 
-    return copy_msg(fin, fout);
+    if (r) {
+	/* got a bad header */
+
+	/* flush the remaining output */
+	copy_msg(fin, fout);
+	/* and return the error */
+	return r;
+    } else {
+	return copy_msg(fin, fout);
+    }
 }
 
 /*
@@ -909,8 +978,8 @@ static int savemsg(struct clientdata *cd,
     time_t t;
     char datestr[80];
 
-    /* Copy to temp file */
-    f = tmpfile();
+    /* Copy to spool file */
+    f = func->spoolfile(m);
     if (!f) {
 	prot_printf(cd->pout, 
 		    "451 4.3.%c cannot create temporary file: %s\r\n",
@@ -975,6 +1044,7 @@ static int savemsg(struct clientdata *cd,
     r = fill_cache(cd->pin, f, m);
     if (r) {
 	fclose(f);
+	func->removespool(m);
 	while (nrcpts--) {
 	    send_lmtp_error(cd->pout, r);
 	}
@@ -1013,6 +1083,7 @@ static int savemsg(struct clientdata *cd,
 		   error_message(errno));
 	}
 	fclose(f);
+	func->removespool(m);
 	return IMAP_IOERROR;
     }
 
@@ -1023,6 +1094,7 @@ static int savemsg(struct clientdata *cd,
 			error_message(errno));
 	}
 	fclose(f);
+	func->removespool(m);
 	return IMAP_IOERROR;
     }
     m->size = sbuf.st_size;
@@ -1340,7 +1412,7 @@ void lmtpmode(struct lmtp_func *func,
 		  inlen = 0;
 	      } else {
 		  unsigned len = strlen(p);
-		  in = xmalloc(len);
+		  in = xmalloc(len+1);
 		  r = sasl_decode64(p, len, in, len, &inlen);
 		  if (r != SASL_OK) {
 		      prot_printf(pout,
@@ -1389,7 +1461,7 @@ void lmtpmode(struct lmtp_func *func,
 		  }
 		  
 		  len = strlen(buf);
-		  in = xmalloc(len);
+		  in = xmalloc(len+1);
 		  r = sasl_decode64(buf, len, in, len, &inlen);
 		  if (r != SASL_OK) {
 		      prot_printf(pout,
@@ -1466,7 +1538,9 @@ void lmtpmode(struct lmtp_func *func,
 		}
 		/* copy message from input to msg structure */
 		r = savemsg(&cd, func, msg);
-		if (r) continue;
+		if (r) {
+		    goto rset;
+		}
 
 		if (msg->size > max_msgsize) {
 		    prot_printf(pout, 
