@@ -1,5 +1,5 @@
 /* mailbox.c -- Mailbox manipulation routines
- $Id: mailbox.c,v 1.119 2002/02/19 18:50:13 ken3 Exp $
+ $Id: mailbox.c,v 1.125 2002/04/29 20:00:23 rjs3 Exp $
  
  * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -94,7 +94,9 @@
 #include "mboxlist.h"
 #include "acapmbox.h"
 #include "seen.h"
+#if 0
 #include "acappush.h"
+#endif
 
 static int mailbox_doing_reconstruct = 0;
 #define zeromailbox(m) { memset(&m, 0, sizeof(struct mailbox)); \
@@ -103,6 +105,7 @@ static int mailbox_doing_reconstruct = 0;
                          (m).cache_fd = -1; }
 
 static int mailbox_calculate_flagcounts(struct mailbox *mailbox);
+static int mailbox_upgrade_index(struct mailbox *mailbox);
 
 /*
  * Names of the headers we cache in the cyrus.cache file.
@@ -120,10 +123,12 @@ char *mailbox_cache_header_name[] = {
 int mailbox_num_cache_header =
   sizeof(mailbox_cache_header_name)/sizeof(char *);
 
+#if 0
 /* acappush variables */
 static int acappush_sock = -1;
 static struct sockaddr_un acappush_remote;
 static int acappush_remote_len = 0;
+#endif
 
 /* function to be used for notification of mailbox changes/updates */
 static mailbox_notifyproc_t *updatenotifier = NULL;
@@ -141,6 +146,7 @@ void mailbox_set_updatenotifier(mailbox_notifyproc_t *notifyproc)
  */
 int mailbox_initialize(void)
 {
+#if 0
     int s;
     int fdflags;
     struct stat sbuf;
@@ -170,6 +176,7 @@ int mailbox_initialize(void)
     if (fdflags == -1) { close(s); return IMAP_IOERROR; }
 
     acappush_sock = s;
+#endif
 
     return 0;
 }
@@ -336,6 +343,32 @@ int mailbox_open_header_path(const char *name,
     return 0;
 }
 
+int mailbox_open_locked(const char *mbname,
+			const char *mbpath,
+			const char *mbacl,
+			struct auth_state *auth_state,
+			struct mailbox *mb,
+			int suppresslog) 
+{
+    int r;
+    
+    r = mailbox_open_header_path(mbname, mbpath, mbacl, auth_state,
+				 mb, suppresslog);
+    if(r) return r;
+
+    /* now we have to close the mailbox if we fail */
+
+    r = mailbox_lock_header(mb);
+    if(!r)
+	r = mailbox_open_index(mb);
+    if(!r) 
+	r = mailbox_lock_index(mb);
+
+    if(r) mailbox_close(mb);
+
+    return r;
+}
+
 #define MAXTRIES 60
 
 /*
@@ -617,6 +650,7 @@ int mailbox_read_acl(struct mailbox *mailbox,
 int mailbox_read_index_header(struct mailbox *mailbox)
 {
     struct stat sbuf;
+    int upgrade = 0;
 
     if (mailbox->index_fd == -1) return IMAP_MAILBOX_BADFORMAT;
 
@@ -676,8 +710,7 @@ int mailbox_read_index_header(struct mailbox *mailbox)
 	if (mailbox_calculate_flagcounts(mailbox))
 	    return IMAP_IOERROR;
 	
-	/* things might have been changed out from under us. reread */
-	mailbox_open_index(mailbox); 
+	upgrade = 1;
     } else {
 	mailbox->deleted = 
 	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_DELETED)));
@@ -688,6 +721,23 @@ int mailbox_read_index_header(struct mailbox *mailbox)
 	mailbox->dirty = 0;
     }
 
+    if (mailbox->start_offset < OFFSET_POP3_NEW_UIDL+sizeof(bit32)) {
+	mailbox->pop3_new_uidl = !mailbox->exists;
+	upgrade = 1;
+    }
+    else {
+	mailbox->pop3_new_uidl = !mailbox->exists ||
+	    ntohl(*((bit32 *)(mailbox->index_base+OFFSET_POP3_NEW_UIDL)));
+    }
+
+    if (upgrade) {
+	if (mailbox_upgrade_index(mailbox))
+	    return IMAP_IOERROR;
+
+	/* things might have been changed out from under us. reread */
+	return mailbox_open_index(mailbox); 
+    }
+
     if (!mailbox_doing_reconstruct &&
 	(mailbox->minor_version < MAILBOX_MINOR_VERSION)) {
 	return IMAP_MAILBOX_BADFORMAT;
@@ -1086,6 +1136,7 @@ int mailbox_write_index_header(struct mailbox *mailbox)
 
     if (updatenotifier) updatenotifier(mailbox);
 
+#if 0
     if (acappush_sock != -1) {
 	acapmbdata_t acapdata;
 
@@ -1104,6 +1155,7 @@ int mailbox_write_index_header(struct mailbox *mailbox)
 	    syslog(LOG_ERR, "sending to acappush: %m");
 	}
     }
+#endif
 
     *((bit32 *)(buf+OFFSET_GENERATION_NO)) = mailbox->generation_no;
     *((bit32 *)(buf+OFFSET_FORMAT)) = htonl(mailbox->format);
@@ -1119,6 +1171,7 @@ int mailbox_write_index_header(struct mailbox *mailbox)
     *((bit32 *)(buf+OFFSET_DELETED)) = htonl(mailbox->deleted);
     *((bit32 *)(buf+OFFSET_ANSWERED)) = htonl(mailbox->answered);
     *((bit32 *)(buf+OFFSET_FLAGGED)) = htonl(mailbox->flagged);
+    *((bit32 *)(buf+OFFSET_POP3_NEW_UIDL)) = htonl(mailbox->pop3_new_uidl);
 
     if (mailbox->start_offset < header_size)
 	header_size = mailbox->start_offset;
@@ -1330,11 +1383,10 @@ struct quota *quota;
 }
 
 /*
- * Lock the index file for 'mailbox'.  DON'T Reread index file header if necessary.
+ * Lock the index file for 'mailbox'.
+ * DON'T Reread index file header if necessary.
  */
-int
-mailbox_lock_index_forcalc(mailbox)
-struct mailbox *mailbox;
+static int mailbox_lock_index_for_upgrade(struct mailbox *mailbox)
 {
     char fnamebuf[MAX_MAILBOX_PATH];
     struct stat sbuffd, sbuffile;
@@ -1377,30 +1429,23 @@ struct mailbox *mailbox;
 }
 
 /*
- * Calculate the number of messages in the mailbox with answered/deleted/flagged system
- * flags
+ * Upgrade the index header for 'mailbox'
  */
-
-static int mailbox_calculate_flagcounts(struct mailbox *mailbox)
+static int mailbox_upgrade_index(struct mailbox *mailbox)
 {
     int r;
     unsigned msgno;
-    bit32 numansweredflag = 0;
-    bit32 numdeletedflag = 0;
-    bit32 numflaggedflag = 0;
-    bit32 oldstart_offset, newstart_offset;
-    bit32 newversion;
-    struct stat sbuf;
+    bit32 oldstart_offset;
     char buf[INDEX_HEADER_SIZE];
     char fnamebuf[MAX_MAILBOX_PATH], fnamebufnew[MAX_MAILBOX_PATH];
     FILE *newindex;
     char *fnametail;
     char *bufp;
 
-    /* Lock files and open new index/cache files */
+    /* Lock files and open new index file */
     r = mailbox_lock_header(mailbox);
     if (r) return r;
-    r = mailbox_lock_index_forcalc(mailbox);
+    r = mailbox_lock_index_for_upgrade(mailbox);
     if (r) {
 	mailbox_unlock_header(mailbox);
 	return r;
@@ -1419,71 +1464,36 @@ static int mailbox_calculate_flagcounts(struct mailbox *mailbox)
     newindex = fopen(fnamebuf, "w+");
     if (!newindex) {
 	syslog(LOG_ERR, "IOERROR: creating %s: %m", fnamebuf);
-	mailbox_unlock_pop(mailbox);
-	mailbox_unlock_index(mailbox);
-	mailbox_unlock_header(mailbox);
-	return IMAP_IOERROR;
+	goto fail;
     }
 
-    /* Copy over old headers to new file */
-    memcpy(buf, mailbox->index_base, mailbox->start_offset);
-
-    /* insert new version number */
-    newversion = htonl(MAILBOX_MINOR_VERSION);
-    memcpy(buf+OFFSET_MINOR_VERSION, &newversion, sizeof(bit32));
+    /* change version number */
+    mailbox->minor_version = MAILBOX_MINOR_VERSION;
 
     /* save old start_offset; change start_offset */
     oldstart_offset = mailbox->start_offset;
-    newstart_offset = htonl(INDEX_HEADER_SIZE);
-    memcpy(buf+OFFSET_START_OFFSET, &newstart_offset, sizeof(bit32));
+    mailbox->start_offset = INDEX_HEADER_SIZE;
 
-    fwrite(buf, 1, oldstart_offset, newindex);
+    /* Write the new index header */ 
+    *((bit32 *)(buf+OFFSET_GENERATION_NO)) = mailbox->generation_no;
+    *((bit32 *)(buf+OFFSET_FORMAT)) = htonl(mailbox->format);
+    *((bit32 *)(buf+OFFSET_MINOR_VERSION)) = htonl(mailbox->minor_version);
+    *((bit32 *)(buf+OFFSET_START_OFFSET)) = htonl(mailbox->start_offset);
+    *((bit32 *)(buf+OFFSET_RECORD_SIZE)) = htonl(mailbox->record_size);
+    *((bit32 *)(buf+OFFSET_EXISTS)) = htonl(mailbox->exists);
+    *((bit32 *)(buf+OFFSET_LAST_APPENDDATE)) = htonl(mailbox->last_appenddate);
+    *((bit32 *)(buf+OFFSET_LAST_UID)) = htonl(mailbox->last_uid);
+    *((bit32 *)(buf+OFFSET_QUOTA_MAILBOX_USED)) = htonl(mailbox->quota_mailbox_used);
+    *((bit32 *)(buf+OFFSET_POP3_LAST_LOGIN)) = htonl(mailbox->pop3_last_login);
+    *((bit32 *)(buf+OFFSET_UIDVALIDITY)) = htonl(mailbox->uidvalidity);
+    *((bit32 *)(buf+OFFSET_DELETED)) = htonl(mailbox->deleted);
+    *((bit32 *)(buf+OFFSET_ANSWERED)) = htonl(mailbox->answered);
+    *((bit32 *)(buf+OFFSET_FLAGGED)) = htonl(mailbox->flagged);
+    *((bit32 *)(buf+OFFSET_POP3_NEW_UIDL)) = htonl(mailbox->pop3_new_uidl);
 
-    if (fstat(mailbox->cache_fd, &sbuf) == -1) {
-	syslog(LOG_ERR, "IOERROR: fstating %s: %m", mailbox->name);
-	fatal("can't fstat cache file", EC_OSFILE);
-    }
-    mailbox->cache_size = sbuf.st_size;
-    map_refresh(mailbox->cache_fd, 0, &mailbox->cache_base,
-		&mailbox->cache_len, mailbox->cache_size, 
-		"cache", mailbox->name);
+    fwrite(buf, 1, INDEX_HEADER_SIZE, newindex);
 
-    /* for each message look at the system flags */
-    for (msgno = 1; msgno <= mailbox->exists; msgno++) {
-	bit32 sysflags;
-	
-	bufp = (char *) (mailbox->index_base + mailbox->start_offset +
-			(msgno - 1)*mailbox->record_size);
-
-	/* Sanity check */
-	if (*((bit32 *)(bufp+OFFSET_UID)) == 0) {
-	    syslog(LOG_ERR, "IOERROR: %s zero index record %u/%lu",
-		   mailbox->name, msgno, mailbox->exists);
-	    goto fail;
-	}
-
-	sysflags = ntohl(*((bit32 *)(bufp+OFFSET_SYSTEM_FLAGS)));
-	if (sysflags & FLAG_ANSWERED)
-	    numansweredflag++;
-	if (sysflags & FLAG_DELETED)
-	    numdeletedflag++;
-	if (sysflags & FLAG_FLAGGED)
-	    numflaggedflag++;
-    }
-
-    mailbox->answered = numansweredflag;
-    mailbox->deleted = numdeletedflag;
-    mailbox->flagged = numflaggedflag;
-
-    /* Grow the index header */
-    numansweredflag = htonl(numansweredflag);
-    fwrite(&numansweredflag, sizeof(bit32),1, newindex);
-    numdeletedflag = htonl(numdeletedflag);
-    fwrite(&numdeletedflag, sizeof(bit32),1, newindex);
-    numflaggedflag = htonl(numflaggedflag);
-    fwrite(&numflaggedflag, sizeof(bit32),1, newindex);
-
-    /* Write the rest of new index header same as old */
+    /* Write the rest of new index same as old */
     for (msgno = 1; msgno <= mailbox->exists; msgno++) {
 	bufp = (char *) (mailbox->index_base + oldstart_offset +
 			(msgno - 1)*mailbox->record_size);
@@ -1526,6 +1536,83 @@ static int mailbox_calculate_flagcounts(struct mailbox *mailbox)
     return IMAP_IOERROR;
 }
 
+/*
+ * Calculate the number of messages in the mailbox with
+ * answered/deleted/flagged system flags
+ */
+
+static int mailbox_calculate_flagcounts(struct mailbox *mailbox)
+{
+    int r;
+    unsigned msgno;
+    bit32 numansweredflag = 0;
+    bit32 numdeletedflag = 0;
+    bit32 numflaggedflag = 0;
+    struct stat sbuf;
+    char *bufp;
+
+    /* Lock files */
+    r = mailbox_lock_header(mailbox);
+    if (r) return r;
+    r = mailbox_lock_index_for_upgrade(mailbox);
+    if (r) {
+	mailbox_unlock_header(mailbox);
+	return r;
+    }
+
+    r = mailbox_lock_pop(mailbox);
+    if (r) {
+	mailbox_unlock_index(mailbox);
+	mailbox_unlock_header(mailbox);
+	return r;
+    }
+
+    if (fstat(mailbox->cache_fd, &sbuf) == -1) {
+	syslog(LOG_ERR, "IOERROR: fstating %s: %m", mailbox->name);
+	fatal("can't fstat cache file", EC_OSFILE);
+    }
+    mailbox->cache_size = sbuf.st_size;
+    map_refresh(mailbox->cache_fd, 0, &mailbox->cache_base,
+		&mailbox->cache_len, mailbox->cache_size, 
+		"cache", mailbox->name);
+
+    /* for each message look at the system flags */
+    for (msgno = 1; msgno <= mailbox->exists; msgno++) {
+	bit32 sysflags;
+	
+	bufp = (char *) (mailbox->index_base + mailbox->start_offset +
+			(msgno - 1)*mailbox->record_size);
+
+	/* Sanity check */
+	if (*((bit32 *)(bufp+OFFSET_UID)) == 0) {
+	    syslog(LOG_ERR, "IOERROR: %s zero index record %u/%lu",
+		   mailbox->name, msgno, mailbox->exists);
+	    mailbox_unlock_pop(mailbox);
+	    mailbox_unlock_index(mailbox);
+	    mailbox_unlock_header(mailbox);
+	    return IMAP_IOERROR;
+	}
+
+	sysflags = ntohl(*((bit32 *)(bufp+OFFSET_SYSTEM_FLAGS)));
+	if (sysflags & FLAG_ANSWERED)
+	    numansweredflag++;
+	if (sysflags & FLAG_DELETED)
+	    numdeletedflag++;
+	if (sysflags & FLAG_FLAGGED)
+	    numflaggedflag++;
+    }
+
+    mailbox->answered = numansweredflag;
+    mailbox->deleted = numdeletedflag;
+    mailbox->flagged = numflaggedflag;
+
+    mailbox_unlock_pop(mailbox);
+    mailbox_unlock_index(mailbox);
+    mailbox_unlock_header(mailbox);
+
+    return 0;
+}
+
 /*
  * Perform an expunge operation on 'mailbox'.  If 'iscurrentdir' is nonzero,
  * the current directory is set to the mailbox directory.  If nonzero, the
@@ -1629,7 +1716,7 @@ void *deciderock;
     fwrite(buf, 1, mailbox->start_offset, newindex);
     /* Grow the index header if necessary */
     for (n = mailbox->start_offset; n < INDEX_HEADER_SIZE; n++) {
-	if (n == OFFSET_FLAGGED+3) {
+	if (n == OFFSET_UIDVALIDITY+3) {
 	    putc(1, newindex);
 	}
 	else {
@@ -1782,6 +1869,7 @@ void *deciderock;
     if (numdeleted) {
 	if (updatenotifier) updatenotifier(mailbox);
 
+#if 0
 	if (acappush_sock != -1) {
 	    acapmbdata_t acapdata;
 	    
@@ -1800,6 +1888,7 @@ void *deciderock;
 		syslog(LOG_ERR, "Error sending to acappush: %m");
 	    }
 	}
+#endif
     }
 
     mailbox_unlock_pop(mailbox);
@@ -1953,6 +2042,7 @@ int mailbox_create(const char *name,
     mailbox.deleted = 0;
     mailbox.answered = 0;
     mailbox.flagged = 0;
+    mailbox.pop3_new_uidl = 1;
 
     if (!uniqueid) {
 	mailbox.uniqueid = xmalloc(sizeof(char) * 32);
@@ -1989,7 +2079,7 @@ int mailbox_create(const char *name,
  */
 int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
 {
-    int r;
+    int r, rquota = 0;
     DIR *dirp;
     struct dirent *f;
     char buf[MAX_MAILBOX_PATH];
@@ -1999,7 +2089,7 @@ int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
     r =  mailbox_lock_header(mailbox);
     if (!r && mailbox->index_fd == -1) r = mailbox_open_index(mailbox);
     if (!r) r = mailbox_lock_index(mailbox);
-    if (!r) r = mailbox_lock_quota(&mailbox->quota);
+    if (!r) rquota = mailbox_lock_quota(&mailbox->quota);
     if (r) {
 	mailbox_close(mailbox);
 	return r;
@@ -2007,9 +2097,9 @@ int mailbox_delete(struct mailbox *mailbox, int delete_quota_root)
 
     seen_delete_mailbox(mailbox);
 
-    if (delete_quota_root) {
+    if (delete_quota_root && !rquota) {
 	mailbox_delete_quota(&mailbox->quota);
-    } else {
+    } else if (!rquota) {
 	/* Free any quota being used by this mailbox */
 	if (mailbox->quota.used >= mailbox->quota_mailbox_used) {
 	    mailbox->quota.used -= mailbox->quota_mailbox_used;
@@ -2069,6 +2159,9 @@ static int expungeall(struct mailbox *mailbox __attribute__((unused)),
     return 1;
 }
 
+/* if 'isinbox' is set, we perform the funky RENAME INBOX INBOX.old
+   semantics, regardless of whether or not the name of the mailbox is
+   'user.foo'.*/
 int
 mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl, 
 	       const char *newname, char *newpath, int isinbox, 
@@ -2077,7 +2170,7 @@ mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl,
 {
     int r, r2;
     struct mailbox oldmailbox, newmailbox;
-    int flag, msgno;
+    unsigned int flag, msgno;
     struct index_record record;
     char oldfname[MAX_MAILBOX_PATH], newfname[MAX_MAILBOX_PATH];
     char *oldfnametail, *newfnametail;
@@ -2134,7 +2227,7 @@ mailbox_rename(const char *oldname, const char *oldpath, const char *oldacl,
 	    strcmp(oldmailbox.quota.root, newmailbox.quota.root) != 0) {
 	    if (!r && newmailbox.quota.limit >= 0 &&
 		newmailbox.quota.used + oldmailbox.quota_mailbox_used >
-		newmailbox.quota.limit * QUOTA_UNITS) {
+		((unsigned) newmailbox.quota.limit * QUOTA_UNITS)) {
 		r = IMAP_QUOTA_EXCEEDED;
 	    }
 	}
@@ -2246,7 +2339,7 @@ mailbox_sync(const char *oldname, const char *oldpath, const char *oldacl,
 {
     int r, r2;
     struct mailbox oldmailbox, newmailbox;
-    int flag, oldmsgno, newmsgno;
+    unsigned int flag, oldmsgno, newmsgno;
     struct index_record oldrecord, newrecord;
     char oldfname[MAX_MAILBOX_PATH], newfname[MAX_MAILBOX_PATH];
     char *oldfnametail, *newfnametail;
@@ -2312,7 +2405,7 @@ mailbox_sync(const char *oldname, const char *oldpath, const char *oldacl,
 	    strcmp(oldmailbox.quota.root, newmailbox.quota.root) != 0) {
 	    if (!r && newmailbox.quota.limit >= 0 &&
 		newmailbox.quota.used + oldmailbox.quota_mailbox_used >
-		newmailbox.quota.limit * QUOTA_UNITS) {
+		((unsigned) newmailbox.quota.limit * QUOTA_UNITS)) {
 		r = IMAP_QUOTA_EXCEEDED;
 	    }
 	}
