@@ -40,7 +40,7 @@
  *
  */
 /*
- * $Id: mboxlist.c,v 1.172 2002/03/05 16:18:15 rjs3 Exp $
+ * $Id: mboxlist.c,v 1.184 2002/04/08 21:01:04 rjs3 Exp $
  */
 
 #include <config.h>
@@ -92,12 +92,11 @@ struct db *mbdb;
 
 static int mboxlist_dbopen = 0;
 
-#define HOSTNAME_SIZE 512
-static const char *mupdate_server = NULL;
-
 static int mboxlist_opensubs();
 static void mboxlist_closesubs();
 
+static int mboxlist_rmquota(const char *name, int matchlen, int maycreate,
+			    void *rock);
 static int mboxlist_changequota(const char *name, int matchlen, int maycreate,
 				void *rock);
 
@@ -211,11 +210,17 @@ static int mboxlist_mylookup(const char *name, int *typep,
 	if (pathp) {
 	    if (mbtype & MBTYPE_REMOTE) {
 		*pathp = partition;
+	    } else if (mbtype & MBTYPE_MOVING) {
+		char *part = strchr(partition, '!');
+		
+		if(!part) return IMAP_SYS_ERROR;
+		else part++; /* skip the !, go to the beginning
+				of the partition name */
+		r = mboxlist_getpath(part, name, pathp);
+		if(r) return r;
 	    } else {
 		r = mboxlist_getpath(partition, name, pathp);
-		if (r) {
-		    return r;
-		}
+		if(r) return r;
 	    }
 	}
 
@@ -299,15 +304,29 @@ int mboxlist_findstage(const char *name, char *stagedir)
     return 0;
 }
 
+int mboxlist_update(char *name, int flags, const char *part, const char *acl)
+{
+    int r = 0;
+    char *mboxent = NULL;
+    
+    mboxent = mboxlist_makeentry(flags, part, acl);
+    r = DB->store(mbdb, name, strlen(name), mboxent, strlen(mboxent), NULL);
+    free(mboxent);
+
+    return r;
+}
+
+
 /*
  * Check/set up for mailbox creation
  */
-int
+static int
 mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition, 
 			      int isadmin, char *userid, 
 			      struct auth_state *auth_state, 
 			      char **newacl, char **newpartition,
-			      int RMW, struct txn **tid)
+			      int RMW, int localonly, int force_user_create,
+			      struct txn **tid)
 {
     int r;
     char *p;
@@ -328,6 +347,10 @@ mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition,
     r = mboxname_policycheck(name);
     if (r) return r;
 
+    /* you must be a real admin to create a local-only mailbox */
+    if(!isadmin && localonly) return IMAP_PERMISSION_DENIED;
+    if(!isadmin && force_user_create) return IMAP_PERMISSION_DENIED;
+
     /* User has admin rights over their own mailbox namespace */
     if (mboxname_userownsmailbox(userid, name)) {
 	isadmin = 1;
@@ -410,7 +433,7 @@ mboxlist_mycreatemailboxcheck(char *name, int mbtype, char *partition,
 	
 	acl = xstrdup("");
 	if (!strncmp(name, "user.", 5)) {
-	    if (strchr(name+5, '.')) {
+	    if (!force_user_create && strchr(name+5, '.')) {
 		/* Disallow creating user.X.* when no user.X */
 		free(acl);
 		return IMAP_PERMISSION_DENIED;
@@ -484,7 +507,7 @@ mboxlist_createmailboxcheck(char *name, int mbtype, char *partition,
 {
     return mboxlist_mycreatemailboxcheck(name, mbtype, partition, isadmin,
 					 userid, auth_state, newacl, 
-					 newpartition, 0, NULL);
+					 newpartition, 0, 0, 0, NULL);
 }
 
 /*
@@ -502,8 +525,9 @@ mboxlist_createmailboxcheck(char *name, int mbtype, char *partition,
  */
 
 int mboxlist_createmailbox(char *name, int mbtype, char *partition, 
-				int isadmin, char *userid, 
-				struct auth_state *auth_state)
+			   int isadmin, char *userid, 
+			   struct auth_state *auth_state,
+			   int localonly, int forceuser)
 {
     int r;
     char *acl = NULL;
@@ -525,7 +549,8 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
     /* 2. verify ACL's to best of ability (CRASH: abort) */
     r = mboxlist_mycreatemailboxcheck(name, mbtype, partition, isadmin, 
 				      userid, auth_state, 
-				      &acl, &newpartition, 1, &tid);
+				      &acl, &newpartition, 1, localonly,
+				      forceuser, &tid);
     switch (r) {
     case 0:
 	break;
@@ -535,6 +560,11 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 	goto done;
     }
 
+    if(mbtype & MBTYPE_MOVING) {
+	r = IMAP_MAILBOX_NOTSUPPORTED;
+	goto done;
+    }
+
     if (!(mbtype & MBTYPE_REMOTE)) {
 	/* Get partition's path */
 	sprintf(buf, "partition-%s", newpartition);
@@ -550,8 +580,8 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
     }
 
     /* 4. Create mupdate reservation */
-    if (mupdate_server) {
-	r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+    if (config_mupdate_server && !localonly) {
+	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
 	    syslog(LOG_ERR,
 		   "can not connect to mupdate server for reservation on '%s'",
@@ -620,14 +650,14 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 	}
 
 	/* delete mupdate entry if we made it */
-	if (madereserved == 1 && mupdate_server) {
+	if (madereserved == 1 && config_mupdate_server) {
 	    r2 = mupdate_delete(mupdate_h, name);
 	    if(r2 > 0) {
 		/* Disconnect, reconnect, and retry */
 		syslog(LOG_WARNING,
 		       "MUPDATE: lost connection, retrying");
 		mupdate_disconnect(&mupdate_h);
-		r2 = mupdate_connect(mupdate_server, NULL,
+		r2 = mupdate_connect(config_mupdate_server, NULL,
 				     &mupdate_h, NULL);
 		if(!r2) {
 		    r2 = mupdate_delete(mupdate_h, name);
@@ -652,7 +682,7 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 
     /* 9. set MUPDATE entry as commited (CRASH: commited) */
     /* xxx maybe we should roll back if this fails? */
-    if (!r && mupdate_server) {
+    if (!r && config_mupdate_server && !localonly) {
 	/* commit the mailbox in MUPDATE */
 	sprintf(buf, "%s!%s", config_servername, newpartition);
 	    
@@ -662,7 +692,7 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 	    syslog(LOG_WARNING,
 		   "MUPDATE: lost connection, retrying");
 	    mupdate_disconnect(&mupdate_h);
-	    r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	    r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	    if(!r) {
 		r = mupdate_activate(mupdate_h, name, buf, acl);
 	    }
@@ -673,7 +703,7 @@ int mboxlist_createmailbox(char *name, int mbtype, char *partition,
 	}
     }
 
-    if(mupdate_server && mupdate_h) mupdate_disconnect(&mupdate_h);
+    if(config_mupdate_server && mupdate_h) mupdate_disconnect(&mupdate_h);
 
     if (acl) free(acl);
     if (newpartition) free(newpartition);
@@ -715,9 +745,7 @@ int mboxlist_insertremote(const char *name, int mbtype, const char *host,
 	
 /*
  * Delete a mailbox.
- * Deleting the mailbox user.FOO deletes the user "FOO".  It may only be
- * performed by an admin.  The operation removes the user "FOO"'s 
- * subscriptions and all sub-mailboxes of user.FOO
+ * Deleting the mailbox user.FOO may only be performed by an admin.
  *
  * 1. Begin transaction
  * 2. Verify ACL's
@@ -729,12 +757,12 @@ int mboxlist_insertremote(const char *name, int mbtype, const char *host,
  *
  */
 int mboxlist_deletemailbox(const char *name, int isadmin, char *userid, 
-			   struct auth_state *auth_state, int checkacl)
+			   struct auth_state *auth_state, int checkacl,
+			   int local_only)
 {
     int r;
     char *acl;
     long access;
-    int deleteuser = 0; /* if we are deleting user.<user> */
     struct mailbox mailbox;
     int deletequotaroot = 0;
     char *path;
@@ -756,32 +784,6 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 
 	/* Only admins may delete user */
 	if (!isadmin) { r = IMAP_PERMISSION_DENIED; goto done; }
-
-	r = mboxlist_mylookup(name, NULL, NULL, NULL, &acl, &tid, 1);
-	switch (r) {
-	case 0:
-	    break;
-	case IMAP_AGAIN:
-	    goto retry;	  
-	    break;
-	default:
-	    DB->abort(mbdb, tid);
-	    goto done;
-	    break;
-	}
-	
-	/* Check ACL before doing anything stupid
-	 * We don't have to lie about the error code since we know
-	 * the user is an admin.
-	 */
-	if (checkacl &&
-	    (!(cyrus_acl_myrights(auth_state, acl) & deleteright))) {
-	    r = IMAP_PERMISSION_DENIED;
-	    DB->abort(mbdb, tid);
-	    goto done;
-	}
-	
-	deleteuser = 1;
     }
 
     r = mboxlist_mylookup(name, &mbtype, &path, NULL, &acl, &tid, 1);
@@ -800,7 +802,8 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 
     isremote = mbtype & MBTYPE_REMOTE;
 
-    /* check if user has Delete right */
+    /* check if user has Delete right (we've already excluded non-admins
+     * from deleting a user mailbox) */
     if(checkacl) {
 	access = cyrus_acl_myrights(auth_state, acl);
 	if(!(access & deleteright)) {
@@ -819,9 +822,9 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 
     /* remove from mupdate */
     /* xxx this can lead to inconsistancies if the later stuff fails */
-    if (!r && mupdate_server) {
+    if (!r && !isremote && !local_only && config_mupdate_server) {
 	/* delete the mailbox in MUPDATE */
-	r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
 	    syslog(LOG_ERR,
 		   "can not connect to mupdate server for delete of '%s'",
@@ -860,7 +863,7 @@ int mboxlist_deletemailbox(const char *name, int isadmin, char *userid,
 	}
     }
 
-    if (r || isremote) goto done;
+    if (r || (isremote && !local_only)) goto done;
 
     if (!r) r = mailbox_open_header_path(name, path, acl, 0, &mailbox, 0);
     if (!r) r = mailbox_delete(&mailbox, deletequotaroot);
@@ -933,8 +936,11 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
     /* Check ability to delete old mailbox */
     if (!strcmp(oldname, newname) && !(mbtype & MBTYPE_REMOTE)) {
 	/* Attempt to move mailbox across partition */
-	if (!isadmin || !partition) {	  
-	    r = IMAP_MAILBOX_EXISTS;
+	if (!isadmin) {
+	    r = IMAP_PERMISSION_DENIED;
+	    goto done;
+	} else if (!partition) {	  
+	    r = IMAP_PARTITION_UNKNOWN;
 	    goto done;
 	}
 
@@ -973,6 +979,12 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
+    /* We don't support renaming mailboxes in transit */
+    if(!r && (mbtype & MBTYPE_MOVING)) {
+	r = IMAP_MAILBOX_NOTSUPPORTED;
+	goto done;
+    }
+
     /* Check ability to create new mailbox */
     if (!partitionmove) {
 	if (!strncmp(newname, "user.", 5) && !strchr(newname+5, '.')) {
@@ -982,7 +994,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
 	r = mboxlist_mycreatemailboxcheck(newname, 0, partition, isadmin, 
 					  userid, auth_state, NULL, 
-					  &newpartition, 1, &tid);
+					  &newpartition, 1, 0, 0, &tid);
 	switch (r) {
 	case 0:
 	    break;
@@ -1006,12 +1018,12 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
-    if (!r && !partitionmove && mupdate_server) {
+    if (!r && !partitionmove && config_mupdate_server) {
 	/* Reserve new name in MUPDATE */
 	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
 	sprintf(buf, "%s!%s", config_servername, newpartition);
 	
-	r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
 	    syslog(LOG_ERR,
 		   "can not connect to mupdate server for reservation on '%s'",
@@ -1029,12 +1041,12 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
 	
 	madenew = 1;
-    } else if(!r && partitionmove && mupdate_server) {
+    } else if(!r && partitionmove && config_mupdate_server) {
 	/* commit the update to MUPDATE */
 	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
 	sprintf(buf, "%s!%s", config_servername, newpartition);
 
-	r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
 	    syslog(LOG_ERR,
 		   "can not connect to mupdate server for reservation on '%s'",
@@ -1109,14 +1121,15 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
 	
 	/* unroll mupdate operations if necessary */
-	if (madenew && mupdate_server) {
+	/* xxx where do we unroll if 'partitionmove == 1' ? */
+	if (madenew && config_mupdate_server) {
 	    r2 = mupdate_delete(mupdate_h, newname);
 	    if(r2 > 0) {
 		/* Disconnect, reconnect, and retry */
 		syslog(LOG_WARNING,
 		       "MUPDATE: lost connection, retrying");
 		mupdate_disconnect(&mupdate_h);
-		r2 = mupdate_connect(mupdate_server, NULL,
+		r2 = mupdate_connect(config_mupdate_server, NULL,
 				     &mupdate_h, NULL);
 		if(!r2) {
 		    r2 = mupdate_delete(mupdate_h, newname);
@@ -1142,7 +1155,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
-    if (!r && !partitionmove && mupdate_server) {
+    if (!r && !partitionmove && config_mupdate_server) {
 	/* commit the mailbox in MUPDATE */
 	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
 	sprintf(buf, "%s!%s", config_servername, newpartition);
@@ -1153,7 +1166,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	    syslog(LOG_WARNING,
 		   "MUPDATE: lost connection, retrying");
 	    mupdate_disconnect(&mupdate_h);
-	    r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	    r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	    if(!r) {
 		r = mupdate_activate(mupdate_h, newname, buf, newacl);
 	    }
@@ -1165,7 +1178,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
-    if (!r && !partitionmove && mupdate_server) {
+    if (!r && !partitionmove && config_mupdate_server) {
 	/* delete the old mailbox in MUPDATE */
 	r = mupdate_delete(mupdate_h, oldname);
 	if(r > 0) {
@@ -1173,7 +1186,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	    syslog(LOG_WARNING,
 		   "MUPDATE: lost connection, retrying");
 	    mupdate_disconnect(&mupdate_h);
-	    r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	    r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	    if(!r) {
 		r = mupdate_delete(mupdate_h, oldname);
 	    }
@@ -1184,7 +1197,7 @@ int mboxlist_renamemailbox(char *oldname, char *newname, char *partition,
 	}
     }
 
-    if(mupdate_server) mupdate_disconnect(&mupdate_h);
+    if(config_mupdate_server) mupdate_disconnect(&mupdate_h);
 
     /* free memory */
     if (newacl) free(newacl);	/* we're done with the new ACL */
@@ -1245,6 +1258,12 @@ int mboxlist_setacl(char *name, char *identifier, char *rights,
 	goto done;
     }
 
+    /* Can't do this to an in-transit mailbox */
+    if(mbtype & MBTYPE_MOVING) {
+	r = IMAP_MAILBOX_NOTSUPPORTED;
+	goto done;
+    }
+
     if (!r && !isadmin && !isusermbox) {
 	access = cyrus_acl_myrights(auth_state, acl);
 	if (!(access & ACL_ADMIN)) {
@@ -1338,13 +1357,13 @@ int mboxlist_setacl(char *name, char *identifier, char *rights,
     }
 
     /* 7. Change mupdate entry  */
-    if (!r && mupdate_server) {
+    if (!r && config_mupdate_server) {
         mupdate_handle *mupdate_h = NULL;
 	/* commit the update to MUPDATE */
 	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
 	sprintf(buf, "%s!%s", config_servername, partition);
 
-	r = mupdate_connect(mupdate_server, NULL, &mupdate_h, NULL);
+	r = mupdate_connect(config_mupdate_server, NULL, &mupdate_h, NULL);
 	if(r) {
 	    syslog(LOG_ERR,
 		   "can not connect to mupdate server for reservation on '%s'",
@@ -1896,7 +1915,7 @@ int mboxlist_setquota(const char *root, int newquota)
 	return r;
     }
     /* Can't set quota on a remote mailbox */
-    if (t & MBTYPE_REMOTE) {
+    if (t & (MBTYPE_REMOTE | MBTYPE_MOVING)) {
 	return IMAP_MAILBOX_NOTSUPPORTED;
     }
 
@@ -1926,6 +1945,49 @@ int mboxlist_setquota(const char *root, int newquota)
     return r;
 }
 
+/*
+ *  Remove a quota root
+ */
+int mboxlist_unsetquota(const char *root)
+{
+    char quota_path[MAX_MAILBOX_PATH];
+    char pattern[MAX_MAILBOX_PATH];
+    int fd;
+    int r=0;
+
+    if (!root[0] || root[0] == '.' || strchr(root, '/')
+	|| strchr(root, '*') || strchr(root, '%') || strchr(root, '?')) {
+	return IMAP_MAILBOX_BADNAME;
+    }
+    
+    mailbox_hash_quota(quota_path, root);
+
+    if ((fd = open(quota_path, O_RDWR, 0)) == -1) {
+	/* already unset */
+	return 0;
+    }
+    
+    close(fd);
+
+    /*
+     * Have to remove it from all affected mailboxes
+     */
+    strcpy(pattern, root);
+    strcat(pattern, ".*");
+    
+    /* top level mailbox */
+    mboxlist_rmquota(root, 0, 0, (void *)root);
+    /* submailboxes - we're using internal names here */
+    mboxlist_findall(NULL, pattern, 1, 0, 0, mboxlist_rmquota, (void *)root);
+
+    if(unlink(quota_path) == -1) {
+	syslog(LOG_ERR, "could not unlink %s (%m)", quota_path);
+	r = IMAP_SYS_ERROR;
+    }
+
+    return r;
+}
+
 /*
  * Retrieve internal information, for reconstructing mailboxes file
  */
@@ -1952,6 +2014,58 @@ int access;
     return access|ACL_LOOKUP|ACL_ADMIN|ACL_CREATE;
 }
 
+/*
+ * Helper function to remove the quota root for 'name'
+ */
+static int mboxlist_rmquota(const char *name, int matchlen, int maycreate,
+			    void *rock)
+{
+    int r;
+    struct mailbox mailbox;
+    const char *oldroot = (const char *) rock;
+
+    assert(rock != NULL);
+
+    r = mailbox_open_header(name, 0, &mailbox);
+    if (r) goto error_noclose;
+
+    r = mailbox_lock_header(&mailbox);
+    if (r) goto error;
+
+    r = mailbox_open_index(&mailbox);
+    if (r) goto error;
+
+    r = mailbox_lock_index(&mailbox);
+    if (r) goto error;
+
+    if (mailbox.quota.root) {
+	if (strlen(mailbox.quota.root) != strlen(oldroot)
+	    || strcmp(mailbox.quota.root, oldroot)) {
+	    /* Part of a different quota root */
+	    mailbox_close(&mailbox);
+	    return 0;
+	}
+
+	/* Need to clear the quota root */
+	free(mailbox.quota.root);
+	mailbox.quota.root = NULL;
+
+	r = mailbox_write_header(&mailbox);	
+	if(r) goto error;
+    }
+
+    mailbox_close(&mailbox);
+    return 0;
+
+ error:
+    mailbox_close(&mailbox);
+ error_noclose:
+    syslog(LOG_ERR, "LOSTQUOTA: unable to remove quota root %s for %s: %s",
+	   oldroot, name, error_message(r));
+    
+    return 0;
+}
+
 /*
  * Helper function to change the quota root for 'name' to that pointed
  * to by the static global struct pointer 'mboxlist_newquota'.
@@ -2015,7 +2129,9 @@ static int mboxlist_changequota(const char *name, int matchlen, int maycreate,
  error_noclose:
     syslog(LOG_ERR, "LOSTQUOTA: unable to change quota root for %s to %s: %s",
 	   name, mboxlist_newquota->root, error_message(r));
-    
+
+    /* Note, we're a callback, and it's not a huge tragedy if we
+     * fail, so we don't ever return a failure */
     return 0;
 }
 
@@ -2038,8 +2154,7 @@ void mboxlist_init(int myflags)
 	r = DB->sync();
     }
 
-    mupdate_server = config_getstring("mupdate_server", NULL);
-    if(mupdate_server) {
+    if(config_mupdate_server) {
 	/* We're going to need SASL */
 	r = sasl_client_init(NULL);
 	if(r != 0) {
