@@ -1,6 +1,6 @@
 /* mupdate-client.c -- cyrus murder database clients
  *
- * $Id: mupdate-client.c,v 1.26 2002/02/28 16:39:04 rjs3 Exp $
+ * $Id: mupdate-client.c,v 1.32 2002/04/05 19:52:27 rjs3 Exp $
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -80,13 +80,14 @@ extern sasl_callback_t *mysasl_callbacks(const char *username,
                                          const char *authname,
                                          const char *realm,
                                          const char *password);
+extern void free_callbacks(sasl_callback_t *in);
 
 static sasl_security_properties_t *make_secprops(int min, int max)
 {
   sasl_security_properties_t *ret =
       (sasl_security_properties_t *) xzmalloc(sizeof(sasl_security_properties_t));
 
-  ret->maxbufsize = 4000; /* xxx */
+  ret->maxbufsize = PROT_BUFSIZE;
   ret->min_ssf = config_getint("sasl_minimum_layer", min);	
   ret->max_ssf = config_getint("sasl_maximum_layer", max);
 
@@ -239,6 +240,7 @@ int mupdate_connect(const char *server, const char *port,
 		    sasl_callback_t *cbs)
 {
     mupdate_handle *h = NULL;
+    int local_cbs = 0;
     struct hostent *hp;
     struct servent *sp;
     struct sockaddr_in addr;
@@ -300,6 +302,7 @@ int mupdate_connect(const char *server, const char *port,
     h->sock = s;
 
     if(!cbs) {
+	local_cbs = 1;
 	cbs = mysasl_callbacks(config_getstring("mupdate_username",""),
 			       config_getstring("mupdate_authname",NULL),
 			       config_getstring("mupdate_realm",NULL),
@@ -336,6 +339,7 @@ int mupdate_connect(const char *server, const char *port,
     if(!mechlist) {
 	syslog(LOG_ERR, "no AUTH banner from remote");
 	mupdate_disconnect(handle);
+	free_callbacks(cbs);
 	return MUPDATE_NOAUTH;
     }
     
@@ -343,10 +347,15 @@ int mupdate_connect(const char *server, const char *port,
 	syslog(LOG_ERR, "authentication to remote mupdate server failed");
 	free(mechlist);
 	mupdate_disconnect(handle);
+	free_callbacks(cbs);
 	return MUPDATE_NOAUTH;
     }
 
     free(mechlist);
+
+    /* xxx unclear that this is correct, but it prevents a memory leak */
+    if(local_cbs) free_callbacks(cbs);
+    
     /* SUCCESS */
     return 0;
 
@@ -459,7 +468,33 @@ int mupdate_reserve(mupdate_handle *handle,
     if (ret) {
 	return ret;
     } else if (response != MUPDATE_OK) {
-	return MUPDATE_FAIL;
+	return MUPDATE_FAIL_RESERVE;
+    } else {
+	return 0;
+    }
+}
+
+int mupdate_deactivate(mupdate_handle *handle,
+		       const char *mailbox, const char *server)
+{
+    int ret;
+    int called = 0;
+    enum mupdate_cmd_response response;
+    
+    if (!handle) return MUPDATE_BADPARAM;
+    if (!mailbox || !server) return MUPDATE_BADPARAM;
+    if (!handle->saslcompleted) return MUPDATE_NOAUTH;
+
+    prot_printf(handle->pout,
+		"X%u DEACTIVATE {%d+}\r\n%s {%d+}\r\n%s\r\n",
+		handle->tagn++, strlen(mailbox), mailbox, 
+		strlen(server), server);
+
+    ret = mupdate_scarf(handle, mupdate_scarf_one, &called, 1, &response);
+    if (ret) {
+	return ret;
+    } else if (response != MUPDATE_OK) {
+	return MUPDATE_FAIL_RESERVE;
     } else {
 	return 0;
     }
@@ -553,10 +588,17 @@ int mupdate_find(mupdate_handle *handle, const char *mailbox,
 
     ret = mupdate_scarf(handle, mupdate_find_cb, handle, 1, &response);
 
-    if (!ret && response == MUPDATE_OK) {
+    /* note that the response is still OK even if there was no data returned,
+     * so we have to make sure we actually filled in the data too */
+    if (!ret && response == MUPDATE_OK && handle->mailboxdata_buf.mailbox) {
 	*target = &(handle->mailboxdata_buf);
 	return 0;
+    } else if(!ret && response == MUPDATE_OK) {
+	/* it looked okay, but we didn't get a mailbox */
+	*target = NULL;
+	return MUPDATE_MAILBOX_UNKNOWN;
     } else {
+	/* Something Bad happened */
 	*target = NULL;
 	return ret ? ret : MUPDATE_FAIL;
     }
@@ -655,8 +697,7 @@ int mupdate_scarf(mupdate_handle *handle,
 
 	if(ch != ' ') {
 	    /* We always have a command */
-	    syslog(LOG_ERR, "Protocol error from master: no tag",
-		   handle->tag.s, ch);
+	    syslog(LOG_ERR, "Protocol error from master: no tag");
 	    r = MUPDATE_PROTOCOL_ERROR;
 	    goto done;
 	}
