@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.115 2002/02/15 17:21:15 rjs3 Exp $
+ * $Id: pop3d.c,v 1.119 2002/04/10 18:02:42 leg Exp $
  */
 #include <config.h>
 
@@ -114,7 +114,8 @@ struct mailbox *popd_mailbox = 0;
 struct sockaddr_in popd_localaddr, popd_remoteaddr;
 int popd_haveaddr = 0;
 char popd_clienthost[250] = "[local]";
-struct protstream *popd_out, *popd_in;
+struct protstream *popd_out = NULL;
+struct protstream *popd_in = NULL;
 unsigned popd_exists = 0;
 unsigned popd_highest;
 unsigned popd_login_time;
@@ -250,6 +251,7 @@ static void popd_reset(void)
     prot_flush(popd_out);
     if (popd_in) prot_free(popd_in);
     if (popd_out) prot_free(popd_out);
+    popd_in = popd_out = NULL;
     close(0);
     close(1);
     close(2);
@@ -495,8 +497,10 @@ void fatal(const char* s, int code)
 	exit(recurse_code);
     }
     recurse_code = code;
-    prot_printf(popd_out, "-ERR [SYS/PERM] Fatal error: %s\r\n", s);
-    prot_flush(popd_out);
+    if (popd_out) {
+	prot_printf(popd_out, "-ERR [SYS/PERM] Fatal error: %s\r\n", s);
+	prot_flush(popd_out);
+    }
     shut_down(code);
 }
 
@@ -790,16 +794,29 @@ static void cmdloop(void)
 			 popd_msg[msg].deleted) {
 		    prot_printf(popd_out, "-ERR No such message\r\n");
 		}
+		else if (mboxstruct.pop3_new_uidl) {
+			    prot_printf(popd_out, "+OK %u %lu.%u\r\n", msg, 
+					mboxstruct.uidvalidity,
+					popd_msg[msg].uid);
+		}
 		else {
-		    prot_printf(popd_out, "+OK %u %u\r\n", msg, popd_msg[msg].uid);
+		    /* old uidl format */
+		    prot_printf(popd_out, "+OK %u %u\r\n", 
+				msg, popd_msg[msg].uid);
 		}
 	    }
 	    else {
 		prot_printf(popd_out, "+OK unique-id listing follows\r\n");
 		for (msg = 1; msg <= popd_exists; msg++) {
 		    if (!popd_msg[msg].deleted) {
-			prot_printf(popd_out, "%u %u\r\n", msg, 
-				    popd_msg[msg].uid);
+			if (mboxstruct.pop3_new_uidl) {
+			    prot_printf(popd_out, "%u %lu.%u\r\n", msg, 
+					mboxstruct.uidvalidity,
+					popd_msg[msg].uid);
+			} else {
+			    prot_printf(popd_out, "%u %u\r\n", msg, 
+					popd_msg[msg].uid);
+			}
 		    }
 		}
 		prot_printf(popd_out, ".\r\n");
@@ -1098,8 +1115,10 @@ char *pass;
 	return;
     }
     else {
-	syslog(LOG_NOTICE, "login: %s %s plaintext %s",
-	       popd_clienthost, popd_userid, reply ? reply : "");
+	syslog(LOG_NOTICE, "login: %s %s plaintext%s %s", popd_clienthost,
+	       popd_userid, popd_starttls_done ? "+TLS" : "", 
+	       reply ? reply : "");
+
 	if ((plaintextloginpause = config_getint("plaintextloginpause", 0))!=0) {
 	    sleep(plaintextloginpause);
 	}
