@@ -40,7 +40,7 @@
  */
 
 /*
- * $Id: pop3proxyd.c,v 1.35 2002/02/27 03:47:15 rjs3 Exp $
+ * $Id: pop3proxyd.c,v 1.38 2002/04/23 18:41:33 rjs3 Exp $
  */
 #include <config.h>
 
@@ -111,7 +111,8 @@ char *popd_userid = 0;
 struct sockaddr_in popd_localaddr, popd_remoteaddr;
 int popd_haveaddr = 0;
 char popd_clienthost[250] = "[local]";
-struct protstream *popd_out, *popd_in;
+struct protstream *popd_out = NULL;
+struct protstream *popd_in = NULL;
 int popd_starttls_done = 0;
 int popd_auth_done = 0;
 
@@ -205,7 +206,7 @@ int service_init(int argc, char **argv, char **envp)
 /*
  * run for each accepted connection
  */
-int service_main(int argc, char **argv, char **envp)
+int service_main(int argc, char **argv, char **envp __attribute__((unused)))
 {
     int pop3s = 0;
     int opt;
@@ -337,7 +338,7 @@ void shut_down(int code)
 #ifdef HAVE_SSL
     tls_shutdown_serverengine();
 #endif
-    prot_flush(popd_out);
+    if (popd_out) prot_flush(popd_out);
     exit(code);
 }
 
@@ -351,8 +352,10 @@ void fatal(const char* s, int code)
 	exit(recurse_code);
     }
     recurse_code = code;
-    prot_printf(popd_out, "-ERR [SYS/PERM] Fatal error: %s\r\n", s);
-    prot_flush(popd_out);
+    if (popd_out) {
+	prot_printf(popd_out, "-ERR [SYS/PERM] Fatal error: %s\r\n", s);
+	prot_flush(popd_out);
+    }
     shut_down(code);
 }
 
@@ -814,8 +817,10 @@ void cmd_pass(char *pass)
 	return;
     }
     else {
-	syslog(LOG_NOTICE, "login: %s %s plaintext %s",
-	       popd_clienthost, popd_userid, reply ? reply : "");
+	syslog(LOG_NOTICE, "login: %s %s plaintext%s %s", popd_clienthost,
+	       popd_userid, popd_starttls_done ? "+TLS" : "", 
+	       reply ? reply : "");
+
 	plaintextloginpause = config_getint("plaintextloginpause", 0);
 	if (plaintextloginpause) sleep(plaintextloginpause);
     }
@@ -882,7 +887,7 @@ void cmd_auth(char *arg)
 {
     int sasl_result;
     static struct buf clientin;
-    unsigned clientinlen=0;
+    int clientinlen=0;
     char *authtype;
     const char *serverout;
     unsigned int serveroutlen;
