@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: sendmail-map.c,v 1.3 2001/02/22 19:27:20 ken3 Exp $
+ * $Id: sendmail-map.c,v 1.4 2001/11/27 02:25:00 ken3 Exp $
  */
 
 #include <config.h>
@@ -61,7 +61,7 @@
 #include <signal.h>
 #include <fcntl.h>
 
-#include <sasl.h>
+#include <sasl/sasl.h>
 #include <acap.h>
 #include <db.h>
 
@@ -113,6 +113,7 @@ extern sasl_callback_t *mysasl_callbacks(const char *username,
 					 const char *authname,
 					 const char *realm,
 					 const char *password);
+void free_callbacks(sasl_callback_t *in);
 
 static acap_conn_t *acap_conn;
 static acap_context_t *mycontext;
@@ -144,18 +145,13 @@ int connect_acap(const char *server, const char *user)
 	fatal("couldn't connect to acap server", EC_UNAVAILABLE);
     }
 
-    cb = mysasl_callbacks(user,
-			  config_getstring("acap_authname", user),
-			  config_getstring("acap_realm", NULL),
-			  config_getstring("acap_password", NULL));
-
     authprog = config_getstring("acap_getauth", NULL);
     if (authprog) {
 	system(authprog);
     }
 
     /* probably should setup callbacks here if configured to! */
-    r = sasl_client_init(cb);
+    r = sasl_client_init(NULL);
     if (r != SASL_OK) {
 	syslog(LOG_ERR, "sasl_client_init() failed: %s",
 	       sasl_errstring(r, NULL, NULL));
@@ -165,7 +161,12 @@ int connect_acap(const char *server, const char *user)
     snprintf(acapurl, sizeof(acapurl), "acap://%s@%s/", user, server);
     r = ACAP_NO_CONNECTION;
 
-    r = acap_conn_connect(acapurl, NULL, &acap_conn);
+    cb = mysasl_callbacks(user,
+			  config_getstring("acap_authname", user),
+			  config_getstring("acap_realm", NULL),
+			  config_getstring("acap_password", NULL));
+    r = acap_conn_connect(acapurl, cb, &acap_conn);
+    free_callbacks(cb);
     if (r != ACAP_OK) {
 	syslog(LOG_ERR, "couldn't connect to ACAP server: %s",
 	       error_message(r));
