@@ -36,10 +36,10 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- * $Id: version.h,v 1.112 2002/03/07 20:00:42 leg Exp $
+ * $Id: version.h,v 1.115 2002/04/30 18:06:05 leg Exp $
  */
 
-#define _CYRUS_VERSION "v2.1.3"
+#define _CYRUS_VERSION "v2.1.4"
 
 /* EXTRA_IDENT is a hack to add some version information for which compile
  * was used to build this version (at CMU, but we don't care what you do with
@@ -53,9 +53,10 @@
 #endif
 
 /* CAPABILITIES are now defined here, not including sasl ones */
-#define CAPABILITY_STRING "IMAP4 IMAP4rev1 ACL QUOTA LITERAL+ NAMESPACE " \
-	"UIDPLUS ID NO_ATOMIC_RENAME UNSELECT CHILDREN MULTIAPPEND SORT " \
-	"THREAD=ORDEREDSUBJECT THREAD=REFERENCES"
+#define CAPABILITY_STRING "IMAP4 IMAP4rev1 ACL QUOTA LITERAL+ " \
+	"MAILBOX-REFERRALS NAMESPACE UIDPLUS ID " \
+	"NO_ATOMIC_RENAME UNSELECT " \
+	"CHILDREN MULTIAPPEND SORT THREAD=ORDEREDSUBJECT THREAD=REFERENCES"
 
 /* Values for ID processing */
 enum {
