@@ -1,6 +1,6 @@
 /* imtest.c -- imap test client
  * Tim Martin (SASL implementation)
- * $Id: imtest.c,v 1.65 2002/01/17 21:08:18 ken3 Exp $
+ * $Id: imtest.c,v 1.67 2002/04/22 19:44:19 rjs3 Exp $
  *
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -664,9 +664,11 @@ imt_stat getauthline(char **line, int *linelen)
   unsigned len;
   char *str=(char *) buf;
   
-  str = prot_fgets(str, BUFSIZE, pin);
-  if (str == NULL) imtest_fatal("prot layer failure");
-  printf("S: %s",str);
+  do {
+      str = prot_fgets(str, BUFSIZE, pin);
+      if (str == NULL) imtest_fatal("prot layer failure");
+      printf("S: %s",str);
+  } while(str[0] == '*');      /* Ignore potential untagged responses */
 
   if (!strncasecmp(str, "A01 OK ", 7)) { return STAT_OK; }
   if (!strncasecmp(str, "A01 NO ", 7)) { return STAT_NO; }
@@ -1139,10 +1141,14 @@ void interactive(char *filename)
 	      FD_CLR(0, &read_set);
 	  } else {
 	      count = strlen(buf);
-	      buf[count - 1] = '\r';
-	      buf[count] = '\n';
-	      buf[count + 1] = '\0';
-	      prot_write(pout, buf, count + 1);
+	      /* If we read a full line, translate the newline */
+	      if (buf[count - 1] == '\n') {
+		  buf[count - 1] = '\r';
+		  buf[count] = '\n';
+		  buf[count + 1] = '\0';
+		  count++;
+	      }
+	      prot_write(pout, buf, count);
 	  }
 	  prot_flush(pout);
       } else if (FD_ISSET(sock, &rset)) {
