@@ -49,7 +49,8 @@ enum cyrusdb_ret {
     CYRUSDB_DONE = 1,
     CYRUSDB_IOERROR = -1,
     CYRUSDB_AGAIN = -2,
-    CYRUSDB_EXISTS = -3
+    CYRUSDB_EXISTS = -3,
+    CYRUSDB_INTERNAL = -4
 };
 
 #define cyrusdb_strerror(c) ("cyrusdb error")
@@ -85,8 +86,9 @@ struct cyrusdb_backend {
     /* checkpoints this database environment */
     int (*sync)(void);
 
-    /* archives this database environment into the specified directory */
-    int (*archive)(const char *dirname);
+    /* archives this database environment, and specified databases
+     * into the specified directory */
+    int (*archive)(const char **fnames, const char *dirname);
 
     /* open the specified database in the global environment */
     int (*open)(const char *fname, struct db **ret);
@@ -144,17 +146,23 @@ struct cyrusdb_backend {
 		 struct txn **tid);
     int (*delete)(struct db *db, 
 		  const char *key, int keylen,
-		  struct txn **tid);
+		  struct txn **tid,
+		  int force); /* 1 = ignore not found errors */
     
     int (*commit)(struct db *db, struct txn *tid);
     int (*abort)(struct db *db, struct txn *tid);
 
     int (*dump)(struct db *db, int detail);
+    int (*consistent)(struct db *db);
 };
 
+extern struct cyrusdb_backend *cyrusdb_backends[];
+
 extern struct cyrusdb_backend cyrusdb_db3;
 extern struct cyrusdb_backend cyrusdb_db3_nosync;
 extern struct cyrusdb_backend cyrusdb_flat;
 extern struct cyrusdb_backend cyrusdb_skiplist;
 
+extern int cyrusdb_copyfile(const char *srcname, const char *dstname);
+
 #endif /* INCLUDED_CYRUSDB_H */
