@@ -88,7 +88,8 @@ static DB_ENV *dbenv;
 static int commit_txn(struct db *db, struct txn *tid);
 static int abort_txn(struct db *db, struct txn *tid);
 
-static void db_panic(DB_ENV *dbenv, int errno)
+static void db_panic(DB_ENV *dbenv __attribute__((unused)),
+		     int errno __attribute__((unused)))
 {
     syslog(LOG_CRIT, "DBERROR: critical database situation");
     /* but don't bounce mail */
@@ -106,6 +107,7 @@ static int init(const char *dbdir, int myflags)
     int flags = 0;
     int maj, min, patch;
     char *vstr;
+    static char errpfx[10]; /* needs to be static; bdb doesn't copy */
 
     if (dbinit++) return 0;
 
@@ -152,7 +154,8 @@ static int init(const char *dbdir, int myflags)
     }
 
     dbenv->set_errcall(dbenv, db_err);
-    dbenv->set_errpfx(dbenv, "db3");
+    snprintf(errpfx, sizeof(errpfx), "db%d", DB_VERSION_MAJOR);
+    dbenv->set_errpfx(dbenv, errpfx);
 
 #if 0
     /* XXX should make this value runtime configurable */
@@ -840,7 +843,8 @@ static int delete_nosync(struct db *db,
     return mydelete(db, key, keylen, tid, DB_TXN_NOSYNC, force);
 }
 
-static int mycommit(struct db *db, struct txn *tid, int txnflags)
+static int mycommit(struct db *db __attribute__((unused)),
+		    struct txn *tid, int txnflags)
 {
     int r;
     DB_TXN *t = (DB_TXN *) tid;
@@ -878,7 +882,8 @@ static int commit_nosync(struct db *db, struct txn *tid)
     return mycommit(db, tid, DB_TXN_NOSYNC);
 }
 
-static int abort_txn(struct db *db, struct txn *tid)
+static int abort_txn(struct db *db __attribute__((unused)),
+		     struct txn *tid)
 {
     int r;
     DB_TXN *t = (DB_TXN *) tid;
@@ -918,7 +923,10 @@ struct cyrusdb_backend cyrusdb_db3 =
     &delete,
 
     &commit_txn,
-    &abort_txn
+    &abort_txn,
+    
+    NULL,
+    NULL
 };
 
 struct cyrusdb_backend cyrusdb_db3_nosync = 
