@@ -1,32 +1,51 @@
 /* glob.c -- fast globbing routine using '*', '%', and '?'
  *
- *        Copyright 1998 by Carnegie Mellon University
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
- *                      All Rights Reserved
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appear in all copies and that
- * both that copyright notice and this permission notice appear in
- * supporting documentation, and that the name of CMU not be
- * used in advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
  *
- * CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
- * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
- * CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
- * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  * Author: Chris Newman
  * Start Date: 4/5/93
  */
 /*
- * $Id: glob.c,v 1.18 1999/03/01 21:13:19 tjs Exp $
+ * $Id: glob.c,v 1.22 2000/12/20 22:53:49 leg Exp $
  */
 
+#include <config.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -311,7 +330,8 @@ int glob_test (g, ptr, len, min)
 	do {
 	    /* see if we match to the next '%' or '*' wildcard */
 	    while (*gptr != '*' && *gptr != '%' && ptr != pend
-		   && (*gptr == TOLOWER(*ptr) || (!newglob && *gptr == '?'))) {
+		   && ((unsigned char) *gptr == TOLOWER(*ptr) || 
+			(!newglob && *gptr == '?'))) {
 		++ptr, ++gptr;
 	    }
 	    if (*gptr == '\0' && ptr == pend) break;
@@ -329,7 +349,7 @@ int glob_test (g, ptr, len, min)
 		 * stop at a sep_char unless we're doing "*%"
 		 */
 		ptr = phier;
-		while (ptr != pend && *ghier != TOLOWER(*ptr)
+		while (ptr != pend && (unsigned char) *ghier != TOLOWER(*ptr)
 		       && (*ptr != g->sep_char ||
 			   (!*ghier && gstar && *gstar == '%' && min
 			    && ptr - start < *min))) {
@@ -359,7 +379,8 @@ int glob_test (g, ptr, len, min)
 		    break;
 		}
 		/* look for a match with first char following '*' */
-		while (pstar != pend && *gstar != TOLOWER(*pstar)) ++pstar;
+		while (pstar != pend && 
+		       (unsigned char) *gstar != TOLOWER(*pstar)) ++pstar;
 		if (pstar == pend) {
 		    gptr = gstar;
 		    break;
