@@ -39,7 +39,7 @@
  *
  * derived from chris newman's code */
 
-/* $Id: imapurl.c,v 1.5 2000/11/05 22:11:28 leg Exp $ */
+/* $Id: imapurl.c,v 1.7 2002/02/01 19:43:36 rjs3 Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -47,13 +47,13 @@
 #include <ctype.h>
 
 /* hexadecimal lookup table */
-static char hex[] = "0123456789ABCDEF";
+static const char hex[] = "0123456789ABCDEF";
 
 /* URL unsafe printable characters */
-static char urlunsafe[] = " \"#%&+:;<=>?@[\\]^`{|}";
+static const char urlunsafe[] = " \"#%&+:;<=>?@[\\]^`{|}";
 
 /* UTF7 modified base64 alphabet */
-static char base64chars[] =
+static const char base64chars[] =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+,";
 #define UNDEFINED 64
 
@@ -297,7 +297,7 @@ void imapurl_fromURL(char *server, char *mailbox, const char *src)
 	at = strchr(src, '@');
 	
 	if (at) {
-	    /* we discard the username for now */
+	    /* xxx we discard the username for now */
 	    src = at + 1; 
 	}
 	*se = '\0';
@@ -310,8 +310,10 @@ void imapurl_fromURL(char *server, char *mailbox, const char *src)
     }
 }
 
-void imapurl_toURL(char *dst, const char *server, const char *mailbox)
+void imapurl_toURL(char *dst, const char *server, const char *mailbox,
+		   const char *mechname)
 {
-    sprintf(dst, "imap://%s/", server);
+    if(mechname) sprintf(dst, "imap://;AUTH=%s@%s/",mechname,server);
+    else sprintf(dst, "imap://%s/", server);
     MailboxToURL(dst + strlen(dst), mailbox);
 }
