@@ -1,6 +1,6 @@
 /* imclient.c -- Streaming IMxP client library
  *
- * $Id: imclient.c,v 1.67 2002/02/22 21:56:17 rjs3 Exp $
+ * $Id: imclient.c,v 1.71 2002/04/23 20:47:44 rjs3 Exp $
  *
  * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -195,11 +195,11 @@ static struct imclient_cmdcallback *cmdcallback_freelist;
 
 /* Forward declarations */
 void imclient_write(struct imclient *imclient,
-		    const char *s, unsigned len);
+		    const char *s, size_t len);
 static int imclient_writeastring P((struct imclient *imclient,
 				     const char *str));
 static void imclient_writebase64 P((struct imclient *imclient,
-				    const char *output, unsigned len));
+				    const char *output, size_t len));
 static void imclient_eof P((struct imclient *imclient));
 static int imclient_decodebase64 P((char *input));
 
@@ -996,7 +996,7 @@ void imclient_getselectinfo(struct imclient *imclient, int *fd,
  */
 void imclient_processoneevent(struct imclient *imclient)
 {
-    char buf[4+IMCLIENT_BUFSIZE];
+    char buf[IMCLIENT_BUFSIZE];
     int n;
     int writelen;
     fd_set rfds, wfds;
@@ -1150,9 +1150,9 @@ static sasl_security_properties_t *make_secprops(int min,int max)
   sasl_security_properties_t *ret=
       (sasl_security_properties_t *)xzmalloc(sizeof(sasl_security_properties_t));
 
-  ret->maxbufsize=1024;
-  ret->min_ssf=min;
-  ret->max_ssf=max;
+  ret->maxbufsize = IMCLIENT_BUFSIZE;
+  ret->min_ssf = min;
+  ret->max_ssf = max;
 
   return ret;
 }
@@ -1409,6 +1409,13 @@ int imclient_authenticate(struct imclient *imclient,
 	}
     } while ((r != 0) && (mtried));
 
+    if (r == 0) {
+	const int *ptr;
+
+	sasl_getprop(imclient->saslconn, SASL_MAXOUTBUF, (const void **) &ptr);
+	imclient->maxplain = *ptr < IMCLIENT_BUFSIZE ? *ptr : IMCLIENT_BUFSIZE;
+    }
+
     free(mlist);
 
     return r;
