@@ -41,7 +41,7 @@
  *
  */
 /*
- * $Id: prot.c,v 1.65 2002/02/19 18:48:10 rjs3 Exp $
+ * $Id: prot.c,v 1.69 2002/04/08 20:01:57 leg Exp $
  */
 
 #include <config.h>
@@ -80,7 +80,7 @@ int write;
 
     newstream = (struct protstream *) xmalloc(sizeof(struct protstream));
     newstream->buf = (unsigned char *) 
-	xmalloc(sizeof(char) * (PROT_BUFSIZE + 4));
+	xmalloc(sizeof(char) * (PROT_BUFSIZE));
     newstream->buf_size = PROT_BUFSIZE;
     newstream->ptr = newstream->buf;
     newstream->cnt = write ? PROT_BUFSIZE : 0;
@@ -111,6 +111,7 @@ int write;
  */
 int prot_free(struct protstream *s)
 {
+    if (s->error) free(s->error);
     free(s->buf);
     free((char*)s);
     return 0;
@@ -119,9 +120,7 @@ int prot_free(struct protstream *s)
 /*
  * Set the logging file descriptor for stream 's' to be 'fd'.
  */
-int prot_setlog(s, fd)
-struct protstream *s;
-int fd;
+int prot_setlog(struct protstream *s, int fd)
 {
     s->logfd = fd;
     return 0;
@@ -135,9 +134,9 @@ int fd;
 
 int prot_settls(struct protstream *s, SSL *tlsconn)
 {
-  s->tls_conn = tlsconn;
+    s->tls_conn = tlsconn;
 
-  return 0;
+    return 0;
 }
 
 #endif /* HAVE_SSL */
@@ -150,46 +149,48 @@ int prot_setsasl(s, conn)
 struct protstream *s;
 sasl_conn_t *conn;
 {
-  const int *ssfp;
-  int result;
-
-  /* flush first if need be */
-  if (s->write && s->ptr != s->buf) prot_flush(s);
-
-  s->conn=conn;
-
-  result = sasl_getprop(conn, SASL_SSF, (const void **) &ssfp);
-  if (result != SASL_OK)
-    return 1;
-
-  s->saslssf = *ssfp;
-
-  if (s->write) {
+    const int *ssfp;
     int result;
-    const int *maxp;
-    int max;
 
-    /* ask SASL for layer max */
-    result = sasl_getprop(conn, SASL_MAXOUTBUF, (const void **) &maxp);
-    max = *maxp;
-    if (result != SASL_OK)
-      return 1;
-
-    if (max == 0 || max > PROT_BUFSIZE) {
-	/* max = 0 means unlimited, and we can't go bigger */
-	max = PROT_BUFSIZE;
+    if (s->write && s->ptr != s->buf) {
+	/* flush any pending output */
+	prot_flush(s);
     }
+   
+    s->conn = conn;
+
+    result = sasl_getprop(conn, SASL_SSF, (const void **) &ssfp);
+    if (result != SASL_OK) {
+	return -1;
+    }
+    s->saslssf = *ssfp;
+
+    if (s->write) {
+	int result;
+	const int *maxp;
+	int max;
+
+	/* ask SASL for layer max */
+	result = sasl_getprop(conn, SASL_MAXOUTBUF, (const void **) &maxp);
+	max = *maxp;
+	if (result != SASL_OK) {
+	    return -1;
+	}
+
+	if (max == 0 || max > PROT_BUFSIZE) {
+	    /* max = 0 means unlimited, and we can't go bigger */
+	    max = PROT_BUFSIZE;
+	}
     
-    max-=50; /* account for extra foo incurred from layers */
+	s->maxplain = max;
+	s->cnt = max;
+    }
+    else if (s->cnt) {  
+	/* flush any pending input */
+	s->cnt = 0;
+    }
 
-    s->maxplain=max;
-    s->cnt=max;
-  }
-  else if (s->cnt) {  /* XXX what does this do? */
-    s->cnt = 0;
-  }
-
-  return 0;
+    return 0;
 }
 
 /*
@@ -307,7 +308,7 @@ int prot_rewind(struct protstream *s)
     assert(!s->write);
 
     if (lseek(s->fd, 0L, 0) == -1) {
-	s->error = strerror(errno);
+	s->error = xstrdup(strerror(errno));
 	return EOF;
     }
     s->cnt = 0;
@@ -350,8 +351,15 @@ int prot_fill(struct protstream *s)
 	    timeout.tv_sec = timeout.tv_usec = 0;
 	    FD_ZERO(&rfds);
 	    FD_SET(s->fd, &rfds);
-	    if (select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
-		       &timeout) <= 0) {
+#ifdef HAVE_SSL
+	    /* maybe there's data stuck in the SSL buffer? */
+	    if (s->tls_conn != NULL) {
+		haveinput = SSL_pending(s->tls_conn);
+	    }
+#endif
+	    if (!haveinput &&
+		(select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
+			&timeout) <= 0)) {
 		if (s->readcallback_proc) {
 		    (*s->readcallback_proc)(s, s->readcallback_rock);
 		    s->readcallback_proc = 0;
@@ -396,7 +404,7 @@ int prot_fill(struct protstream *s)
 		     (now < read_timeout));
 	    if (r == 0) {
 		if (!s->dontblock) {
-		    s->error = "idle for too long";
+		    s->error = xstrdup("idle for too long");
 		    return EOF;
 		} else {
 		    errno = EAGAIN;
@@ -419,7 +427,7 @@ int prot_fill(struct protstream *s)
 	} while (n == -1 && errno == EINTR);
 		
 	if (n <= 0) {
-	    if (n) s->error = strerror(errno);
+	    if (n) s->error = xstrdup(strerror(errno));
 	    else s->eof = 1;
 	    return EOF;
 	}
@@ -428,20 +436,25 @@ int prot_fill(struct protstream *s)
 	    int result;
 	    const char *out;
 	    unsigned outlen;
-	    static char errbuf[256];
 	    
 	    /* Decode the input token */
 	    result = sasl_decode(s->conn, (const char *) s->buf, n, 
 				 &out, &outlen);
 	    
 	    if (result != SASL_OK) {
-		snprintf(errbuf, 256, "Decoding error: %s (%i)",
-			 sasl_errstring(result, NULL, NULL), result);
-		s->error = errbuf;
+		char errbuf[256];
+		const char *ed = sasl_errdetail(s->conn);
+
+		snprintf(errbuf, 256, "decoding error: %s; %s",
+			 sasl_errstring(result, NULL, NULL),
+			 ed ? ed : "no detail");
+		s->error = xstrdup(errbuf);
 		return EOF;
 	    }
 	    
 	    if (outlen > 0) {
+		/* XXX can we just serve data from 'out' without copying
+		   it to s->buf ? */
 		if (outlen > s->buf_size) {
 		    s->buf = (unsigned char *) 
 			xrealloc(s->buf, sizeof(char) * (outlen + 4));
@@ -532,14 +545,20 @@ int prot_flush(struct protstream *s)
     }
 
     if (s->saslssf != 0) {
-	/* Encode the data */  /* xxx handle left */
+	/* encode the data */
 	unsigned int outlen;
 	int result;
 	
 	result = sasl_encode(s->conn, (char *) ptr, left, 
 			     &encoded_output, &outlen);
 	if (result != SASL_OK) {
-	    s->error = "Encoding error";
+	    char errbuf[256];
+	    const char *ed = sasl_errdetail(s->conn);
+	    
+	    snprintf(errbuf, 256, "encoding error: %s; %s",
+		     sasl_errstring(result, NULL, NULL),
+		     ed ? ed : "no detail");
+	    s->error = xstrdup(errbuf);
 	    return EOF;
 	}
 	
@@ -559,7 +578,7 @@ int prot_flush(struct protstream *s)
 	n = write(s->fd, ptr, left);
 #endif /* HAVE_SSL */
 	if (n == -1 && errno != EINTR) {
-	    s->error = strerror(errno);
+	    s->error = xstrdup(strerror(errno));
 	    return EOF;
 	}
 
@@ -581,10 +600,11 @@ int prot_flush(struct protstream *s)
  */
 int prot_write(struct protstream *s, const char *buf, unsigned len)
 {
-    assert(len >= 0);
     assert(s->write);
 
     while (len >= s->cnt) {
+	/* XXX can we manage to write data from 'buf' without copying it
+	   to s->ptr ? */
 	memcpy(s->ptr, buf, s->cnt);
 	s->ptr += s->cnt;
 	buf += s->cnt;
