@@ -40,7 +40,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: notify_mailto.c,v 1.2 2002/03/01 20:32:38 ken3 Exp $
+ * $Id: notify_mailto.c,v 1.3 2002/03/18 15:14:17 ken3 Exp $
  */
 
 #include "notify_mailto.h"
@@ -65,8 +65,11 @@
 
 static int global_outgoing_count = 0;
 
-char* notify_mailto(int nopt, char **options,
+char* notify_mailto(const char *class __attribute__((unused)),
 		    const char *priority __attribute__((unused)),
+		    const char *user __attribute__((unused)),
+		    const char *mailbox __attribute__((unused)),
+		    int nopt, char **options,
 		    const char *message)
 {
     FILE *sm;
