@@ -43,7 +43,7 @@ use strict;
 use Cyrus::IMAP;
 use vars qw($VERSION
 	    *create *delete *deleteacl *listacl *list *rename *setacl
-	    *subscribed *quota *quotaroot);
+	    *subscribed *quota *quotaroot *info);
 
 $VERSION = '1.00';
 
@@ -74,10 +74,13 @@ sub new {
   # but the API makes it difficult to get at the results of that command.
   if(defined($self)) {
     $self->{support_referrals} = 0;
+    $self->{support_annotatatemore} = 0;
     $self->addcallback({-trigger => 'CAPABILITY',
 			-callback => sub {my %a = @_;
 					  map { $self->{support_referrals} = 1
 						  if /^MAILBOX-REFERRALS$/i;
+						$self->{support_annotatemore} = 1
+						  if /^ANNOTATEMORE$/i;
 					      }
 					  split(/ /, $a{-text})}});
     $self->send(undef, undef, 'CAPABILITY');
@@ -383,9 +386,10 @@ sub listquota {
       $cyradm->authenticate()
 	or die "cyradm: cannot authenticate to $refserver\n";
 
-      my $ret = $cyradm->listquota($root);
+      my @ret = $cyradm->listquota($root);
+      $self->{error} = $cyradm->{error};
       $cyradm = undef;
-      return $ret;
+      return @ret;
     } else {
       $self->{error} = $msg;
       ();
@@ -436,9 +440,10 @@ sub listquotaroot {
       $cyradm->authenticate()
 	or die "cyradm: cannot authenticate to $refserver\n";
       
-      my $ret = $cyradm->listquotaroot($root);
+      my @ret = $cyradm->listquotaroot($root);
+      $self->{error} = $cyradm->{error};
       $cyradm = undef;
-      return $ret;
+      return @ret;
     } else {
       $self->{error} = $msg;
       ();
@@ -497,6 +502,7 @@ my %aclalias = (none => '',
 		post => 'lrsp',
 		append => 'lrsip',
 		write => 'lrswipcd',
+		delete => 'lrd',
 		all => 'lrswipcda');
 
 sub setaclmailbox {
@@ -510,8 +516,35 @@ sub setaclmailbox {
     if ($rc eq 'OK') {
       $cnt++;
     } else {
-      $res .= "\n" if $res ne '';
-      $res .= $id . ': ' . $acl{$id} . ': ' . $msg;
+      if($self->{support_referrals} && $msg =~ m|^\[REFERRAL\s+([^\]\s]+)\]|) {
+	my ($refserver, $box) = Cyrus::IMAP->fromURL($1);
+	my $port = 143;
+
+	if($refserver =~ /:/) {
+	  $refserver =~ /([^:]+):(\d+)/;
+	  $refserver = $1; $port = $2;
+	}
+
+	my $cyradm = Cyrus::IMAP::Admin->new($refserver, $port)
+	  or die "cyradm: cannot connect to $refserver\n";
+	$cyradm->addcallback({-trigger => 'EOF',
+			      -callback => \&_cb_ref_eof,
+			      -rock => \$cyradm});
+	$cyradm->authenticate()
+	  or die "cyradm: cannot authenticate to $refserver\n";
+
+	my $ret = $cyradm->setaclmailbox($mbx, %acl);
+	if(defined($ret)) {
+	  $cnt++;
+	  $rc = 'OK';
+	} else {
+	  $res .= "\n" if $res ne '';
+	  $res .= $id . ': ' . $acl{$id} . ': ' . $cyradm->{error};
+	}
+      } else {
+	$res .= "\n" if $res ne '';
+	$res .= $id . ': ' . $acl{$id} . ': ' . $msg;
+      }
     }
   }
   if ($rc eq 'OK') {
@@ -568,6 +601,70 @@ sub setquota {
   }
 }
 
+# xxx this is entirely untested ;)
+sub getinfo {
+  my ($self,$box) = @_;
+  my $pat;
+
+  if($box == undef) {
+    $pat = "/server/*";
+  } else {
+    $pat = "/mailbox/$box/*";
+  }
+
+  if(!$self->{support_annotatemore}) {
+    $self->{error} = "Remote does not support ANNOTATEMORE.";
+    return undef;
+  }
+
+  my %info = ();
+  $self->addcallback({-trigger => 'ANNOTATION',
+		      -callback => sub {
+			my %d = @_;
+			my $text = $d{-text};
+			my $annotation;
+
+			if($text =~ /^\([^\)]*\)$/) {
+			  # list of annotations
+			  $text =~ s/^\(//;
+			  
+			  while($text !~ /^\)/) {
+			    if($text =~
+			       /^\s*\"([^\"]*)\"\s+\(\"([^\"]*)\"\s+\"([^\"]*)\"\)/) {
+				 ${$d{-rock}}{$1} = $3;
+				 $text =~ s/^\s*\"([^\"]*)\"\s+\(\"([^\"]*)\"\s+\"([^\"]*)\"\)//;
+			       } else {
+				 # hrm, error
+				 $self->{error} = "Could not parse";
+				 return undef;
+			       }
+			  }
+			} elsif ($text =~ /^\"([^\"]+)\"/) {
+			  # Single annotation, but possibly multiple values
+			  # however, we are only asking for one value, so...
+			  $annotation = $1;
+			  $text =~ /\"(\S*)\"\)/;
+			  ${$d{-rock}}{$annotation} = $1;
+		        } else {
+			  next;
+			}
+		      },
+		      -rock => \%info});
+
+  # send getannotation "/mailbox/name/* or /server/*"
+  my ($rc, $msg) = $self->send('', '', "GETANNOTATION %s \"value.shared\"",
+			       $pat);
+  $self->addcallback({-trigger => 'ANNOTATION'});
+  if ($rc eq 'OK') {
+    $self->{error} = undef;
+    @info;
+  } else {
+    $self->{error} = $msg;
+    ();
+  }
+}
+*info = *getinfo;
+
 sub error {
   my $self = shift;
   $self->{error};
@@ -700,7 +797,8 @@ Renames the specified mailbox, optionally moving it to a different partition.
 
 Set ACLs on a mailbox.  The ACL may be one of the special strings C<none>,
 C<read> (C<lrs>), C<post> (C<lrsp>), C<append> (C<lrsip>), C<write>
-(C<lrswipcd>), or C<all> (C<lrswipcda>), or any combinations of the ACL codes:
+(C<lrswipcd>), C<delete> (C<lrd>), or C<all> (C<lrswipcda>), or any combinations 
+of the ACL codes:
 
 =over 4
 
