@@ -43,6 +43,10 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/file.h>
@@ -61,7 +65,6 @@
 
 #include <prot.h>
 
-
 struct iseive_s {
     char *serverFQDN;
     int port;
@@ -77,6 +80,8 @@ struct iseive_s {
     struct protstream *pout;
 };
 
+void fillin_interactions(sasl_interact_t *tlist);
+
 /* we need this separate from the free() call so that we can reuse
  * the same memory for referrals */
 static void sieve_dispose(isieve_t *obj) 
@@ -160,14 +165,19 @@ int init_sasl(isieve_t *obj,
 	      int ssf,
 	      sasl_callback_t *callbacks)
 {
-  int saslresult;
+  static int sasl_started = 0;
+  int saslresult = SASL_OK;
   sasl_security_properties_t *secprops=NULL;
   socklen_t addrsize=sizeof(struct sockaddr_in);
   struct sockaddr_in saddr_l, saddr_r;
   char localip[60], remoteip[60];
 
   /* attempt to start sasl */
-  saslresult=sasl_client_init(NULL);
+  if(!sasl_started) {
+      saslresult=sasl_client_init(NULL);
+      obj->conn = NULL;
+      sasl_started = 1;
+  }
 
   /* Save the callbacks array */
   obj->callbacks = callbacks;
@@ -193,6 +203,8 @@ int init_sasl(isieve_t *obj,
 		 sizeof(struct sockaddr_in), localip, 60))
       return -1;
 
+  if(obj->conn) sasl_dispose(&obj->conn);
+
   /* client new connection */
   saslresult=sasl_client_new(SIEVE_SERVICE_NAME,
 			     obj->serverFQDN,
@@ -321,7 +333,8 @@ static int getauthline(isieve_t *obj, char **line, unsigned int *linelen,
 }
 
 
-int auth_sasl(char *mechlist, isieve_t *obj, char **errstr)
+int auth_sasl(char *mechlist, isieve_t *obj, const char **mechusing,
+	      char **errstr)
 {
   sasl_interact_t *client_interact=NULL;
   int saslresult=SASL_INTERACT;
@@ -329,29 +342,29 @@ int auth_sasl(char *mechlist, isieve_t *obj, char **errstr)
   unsigned int outlen;
   char *in;
   unsigned int inlen;
-  const char *mechusing;
   char inbase64[2048];
   unsigned int inbase64len;
 
   imt_stat status = STAT_CONT;
 
+  if(!mechlist || !obj || !mechusing) return -1;
+
   /* call sasl client start */
   while (saslresult==SASL_INTERACT)
   {
     saslresult=sasl_client_start(obj->conn, mechlist,
 				 &client_interact,
 				 &out, &outlen,
-				 &mechusing);
+				 mechusing);
     if (saslresult==SASL_INTERACT)
       fillin_interactions(client_interact); /* fill in prompts */      
-
   }
 
   if ((saslresult!=SASL_OK) && (saslresult!=SASL_CONTINUE)) return saslresult;
 
   if (out!=NULL)
   {
-    prot_printf(obj->pout,"AUTHENTICATE \"%s\" ",mechusing);
+    prot_printf(obj->pout,"AUTHENTICATE \"%s\" ",*mechusing);
 
     sasl_encode64(out, outlen,
 		  inbase64, sizeof(inbase64), &inbase64len);
@@ -360,12 +373,14 @@ int auth_sasl(char *mechlist, isieve_t *obj, char **errstr)
     prot_write(obj->pout,inbase64,inbase64len);
     prot_printf(obj->pout,"\r\n");
   } else {
-    prot_printf(obj->pout,"AUTHENTICATE \"%s\"\r\n",mechusing);
+    prot_printf(obj->pout,"AUTHENTICATE \"%s\"\r\n",*mechusing);
   }
   prot_flush(obj->pout);
 
   inlen = 0;
-  status = getauthline(obj,&in,&inlen, errstr);
+
+  /* get reply */
+  status=getauthline(obj,&in,&inlen, errstr);
 
   while (status==STAT_CONT)
   {
@@ -385,7 +400,17 @@ int auth_sasl(char *mechlist, isieve_t *obj, char **errstr)
     /* check if sasl suceeded */
     if (saslresult<SASL_OK)
     {
-	*errstr = strdup(sasl_errstring(saslresult,NULL,NULL));
+	/* send cancel notice */
+	prot_printf(obj->pout, "*\r\n");
+	prot_flush(obj->pout);
+
+	/* eat the auth line that confirms that we canceled */
+	if(getauthline(obj,&in,&inlen,errstr) != STAT_NO) {
+	    *errstr = strdup("protocol error");
+	} else {
+	    *errstr = strdup(sasl_errstring(saslresult,NULL,NULL));
+	}
+	
 	return saslresult;
     }
 
@@ -434,7 +459,8 @@ int do_referral(isieve_t *obj, char *refer_to)
     char *mechlist;
     int port;
     char *errstr;
-    
+    const char *mtried;
+        
     /* xxx we're not supporting port numbers here */
 
     serv = getservbyname("sieve", "tcp");
@@ -454,7 +480,7 @@ int do_referral(isieve_t *obj, char *refer_to)
     /* Authenticate */
     mechlist = read_capability(obj_new);
 
-    ret = auth_sasl(mechlist, obj_new, &errstr);
+    ret = auth_sasl(mechlist, obj_new, &mtried, &errstr);
     if(ret) return STAT_NO;
 
     /* free old isieve_t */
@@ -479,18 +505,22 @@ int isieve_logout(isieve_t **obj)
     
     sieve_free_net(*obj);
     *obj = NULL;
+
+    return STAT_OK;
 }
 
-int isieve_put_file(isieve_t *obj, char *filename, char **errstr)
+int isieve_put_file(isieve_t *obj, char *filename, char *destname,
+		    char **errstr)
 {
     char *refer_to;
     int ret = installafile(obj->version,
 			   obj->pout, obj->pin,
-			   filename, &refer_to, errstr);
+			   filename, destname,
+			   &refer_to, errstr);
     if(ret == -2 && refer_to) {
 	ret = do_referral(obj, refer_to);
 	if(ret == STAT_OK) {
-	    ret = isieve_put_file(obj, filename, errstr);
+	    ret = isieve_put_file(obj, filename, destname, errstr);
 	} else {
 	    *errstr = "referral failed";
 	}
