@@ -64,10 +64,11 @@ typedef enum {
     STAT_OK = 2
 } imt_stat;
 
-int auth_sasl(char *mechlist, isieve_t *obj, char **errstr);
+int auth_sasl(char *mechlist, isieve_t *obj, const char **mechusing, char **errstr);
 
 int isieve_logout(isieve_t **obj);
-int isieve_put_file(isieve_t *obj, char *filename, char **errstr);
+int isieve_put_file(isieve_t *obj, char *filename, char *destname,
+		    char **errstr);
 int isieve_put(isieve_t *obj, char *name, char *data, int len, char **errstr);
 int isieve_delete(isieve_t *obj, char *name, char **errstr);
 typedef void *isieve_listcb_t(char *name, int isactive, void *rock);
