@@ -48,12 +48,10 @@
 #endif
 
 #include "mystring.h"
-
 #include "codes.h"
 
 #include <stdlib.h>
-
-#include <strings.h>
+#include <string.h>
 
 int string_allocate(int length,
 		    const char *buf,	/* NULL => no copy */
