@@ -1,6 +1,6 @@
 /* interp.c -- sieve script interpretor builder
  * Larry Greenfield
- * $Id: interp.c,v 1.19 2000/12/12 21:41:26 ken3 Exp $
+ * $Id: interp.c,v 1.21 2002/03/10 02:58:09 ken3 Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -36,8 +36,6 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "sieve_interface.h"
 #include "interp.h"
 
-const char *sieve_version = "cmu-sieve 2.0";
-
 /* build a sieve interpretor */
 int sieve_interp_alloc(sieve_interp_t **interp, void *interp_context)
 {
@@ -62,7 +60,6 @@ int sieve_interp_alloc(sieve_interp_t **interp, void *interp_context)
     i->vacation = NULL;
     i->notify = NULL;
 
-    i->curflags.flag = NULL; i->curflags.nflags = 0;
     i->markflags = NULL;
 
     i->interp_context = interp_context;
@@ -85,18 +82,8 @@ const char *sieve_listextensions(void)
     return sieve_extensions;
 }
 
-void free_imapflags(sieve_imapflags_t *imapflags)
-{
-    while (imapflags->nflags)
-	free(imapflags->flag[--imapflags->nflags]);
-    free(imapflags->flag);
-    
-    imapflags->flag = NULL;
-}
-  
 int sieve_interp_free(sieve_interp_t **interp)
 {
-    free_imapflags(&(*interp)->curflags);
     free(*interp);
     
     return SIEVE_OK;
