@@ -1,6 +1,6 @@
 /* message.h
  * Larry Greenfield
- * $Id: message.h,v 1.12 2002/02/13 20:44:04 rjs3 Exp $
+ * $Id: message.h,v 1.13 2002/02/19 18:09:46 ken3 Exp $
  */
 /***********************************************************
         Copyright 1999 by Carnegie Mellon University
@@ -82,15 +82,16 @@ struct Action {
     int vac_days;
 };
 
-typedef struct notify_action_s {
-
-    int exists; /* 0 = no +/-1 = yes (-1 flags default action) */
-
+typedef struct notify_list_s {
+    int isactive;
+    char *id;
+    char *method;
+    stringlist_t **options;
     const char *priority;
     char *message;
-    stringlist_t *headers;
+    struct notify_list_s *next;
 
-} notify_action_t;
+} notify_list_t;
 
 /* header parsing */
 typedef enum {
@@ -105,14 +106,15 @@ int parse_address(const char *header, void **data, void **marker);
 char *get_address(address_part_t addrpart, void **data, void **marker,
 		  int canon_domain);
 int free_address(void **data, void **marker);
-notify_action_t *default_notify_action(void);
+notify_list_t *new_notify_list(void);
+void free_notify_list(notify_list_t *n);
 
 /* actions; return negative on failure.
  * these don't actually perform the actions, they just add it to the
  * action list */
 int do_reject(action_list_t *m, char *msg);
 int do_fileinto(action_list_t *m, char *mbox, sieve_imapflags_t *imapflags);
-int do_forward(action_list_t *m, char *addr);
+int do_redirect(action_list_t *m, char *addr);
 int do_keep(action_list_t *m, sieve_imapflags_t *imapflags);
 int do_discard(action_list_t *m);
 int do_vacation(action_list_t *m, char *addr, char *fromaddr,
@@ -123,9 +125,11 @@ int do_addflag(action_list_t *m, char *flag);
 int do_removeflag(action_list_t *m, char *flag);
 int do_mark(action_list_t *m);
 int do_unmark(action_list_t *m);
-int do_notify(sieve_interp_t *i,void *m, notify_action_t *notify,
-	      const char *priority, char *message, stringlist_t *sl);
-int do_denotify(notify_action_t *notify);
+int do_notify(notify_list_t *n, char *id,
+	      char *method, stringlist_t **options,
+	      const char *priority, char *message);
+int do_denotify(notify_list_t *n, comparator_t *comp, void *pat,
+		const char *priority);
 
 
 #endif
